/*******************************************************************************
* 
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
* 
* Copyright 2009 Mobilygen Corporation
* All rights reserved.
* 
* QuArc is a registered trademark of Mobilygen Corporation.
* 
*******************************************************************************/

#ifndef __MOBISOCK_H
#define __MOBISOCK_H

#define MOBISOCK_MAX_PACKET_BYTES 256
#define MOBISOCK_MAX_FILENAME     256
#define MOBISOCK_MAX_STREAMS      16

typedef void *MOBISOCK_HANDLE;

typedef enum {
    MOBISOCK_CMD_TYPE_SIGL = 0,  /* signal command  */
    MOBISOCK_CMD_TYPE_ECHO = 1,  /* echo command    */
    MOBISOCK_CMD_TYPE_KILL = 2,  /* kill connection */
    MOBISOCK_CMD_TYPE_COPY = 3,
    MOBISOCK_CMD_TYPE_DEL  = 4,
    MOBISOCK_CMD_TYPE_RECOVER = 5,

    MOBISOCK_CMD_TYPE_NUM,
} MOBISOCK_CMD_TYPE;

/* The 'base class' of mobisock commands */
typedef struct {
    MOBISOCK_CMD_TYPE type;
} MOBISOCK_CMD;

/* The Echo command */
typedef struct {
    MOBISOCK_CMD_TYPE type;
    char echoStr[MOBISOCK_MAX_PACKET_BYTES];
    int len; // length of the echoStr
} MOBISOCK_CMD_ECHO;

/* The power recovery command */
typedef struct {
    MOBISOCK_CMD_TYPE type;
    char filename[MOBISOCK_MAX_FILENAME];
    int nlen;
} MOBISOCK_CMD_RECOVER;

/* The copy or delete over a time range command */
typedef struct {
    MOBISOCK_CMD_TYPE type;
    char path[MOBISOCK_MAX_FILENAME];
    char name[MOBISOCK_MAX_FILENAME];
    char cpyFileName[MOBISOCK_MAX_FILENAME]; // for copy
    int numStreams;
    int streamID[MOBISOCK_MAX_STREAMS];
    unsigned int startTimeHi;
    unsigned int startTimeLo;
    unsigned int stopTimeHi;
    unsigned int stopTimeLo;
} MOBISOCK_CMD_FILE_MNG;

typedef struct {
    int size; // size of the buf
    char *buf;
} MOBISOCK_MSG;

MOBISOCK_HANDLE MobiSockAPI_Create();
void MobiSockAPI_Destroy(MOBISOCK_HANDLE h);
int MobiSockAPI_ConnectToMobiApp(MOBISOCK_HANDLE h);
int MobiSockAPI_SendCommand(MOBISOCK_HANDLE h, MOBISOCK_CMD *cmd);
int MobiSockAPI_RecvMessage(MOBISOCK_HANDLE h, MOBISOCK_MSG *msg);
int MobiSockAPI_CloseConnection(MOBISOCK_HANDLE h);

#endif
