--[[
 This script can be used as a base to set up the rate conversion with 
audio decoder. The decoder first decodes the bitstream and then the output 
is binded to the audio filter (select rate conversion module).
The output of the rate conversion module is binded to the AVSN.

The audio filter uses up to 4 audio decoders as the input channels.

--]]

OUTPUT_SEL = OUTPUT_SEL or 1;

INPUT_FILENAME = INPUT_FILENAME or {};

INPUT_FILENAME[1] = INPUT_FILENAME[1] or "/media/sda1/aud/audenc0.qbx";
INPUT_FILENAME[2] = INPUT_FILENAME[2] or "/media/sda1/aud/audenc1.qbx";
INPUT_FILENAME[3] = INPUT_FILENAME[3] or "/media/sda1/aud/audenc2.qbx";
INPUT_FILENAME[4] = INPUT_FILENAME[4] or "/media/sda1/aud/audenc3.qbx";

AUDIO_STREAM_ID = {};
AUDIO_STREAM_ID[1] = 1;
AUDIO_STREAM_ID[2] = 2;
AUDIO_STREAM_ID[3] = 3;
AUDIO_STREAM_ID[4] = 4;

SAMPLERATE = SAMPLERATE or 8000;
OUTSAMPLERATE = 32000;
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_Q711;

AUD_FILTER_TYPE = AUD_FILTER_TYPE or Q_AUDFLTR_CFP_TYPE_RATECONVERSION;
AUD_FILTER_INPUTS = AUD_FILTER_INPUTS or 1;
AUD_FILTER_OUTPUTS = AUD_FILTER_OUTPUTS or 1;

AUD_FILTER_OUT_FREQ = OUTSAMPLERATE;
AUD_FILTER_OUT_FRAMESIZE = 1024;
AUD_FILTER_IN_FREQ = SAMPLERATE;
AUD_FILTER_NUM_CHAN = 1;
AUD_FILTER_IN_FRAMESIZE = 256;

AUDIO_BUFFER_SIZE = 1000000;


if (OUTSAMPLERATE == 8000) then
    board.setAudioSampleFreq({port=0, freq=16000});
    sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=2048000});
    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=2048000});
else
    board.setAudioSampleFreq({port=0, freq=OUTSAMPLERATE});
end


-- create AV decoders
avdec = {};
for i = 1, AUD_FILTER_INPUTS do
   avdec[i] = sysctl0:avdecCreate({name="avdec"..i, atype=AUDIO_CODEC, vtype=Q_AVDEC_CFP_VIDEO_CODEC_NONE, 
				   audioBufferSize=AUDIO_BUFFER_SIZE});
   avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID[i]});
   avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
   avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
   avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
   if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QPCM) then
      avdec[i]:setParam({param=Q_AVDEC_CMP_QPCM_ENDIANNESS, value=Q_AVDEC_CFP_AUDIO_QPCM_BIG_ENDIAN});
   end
   avdec[i]:activateCfg();
end

-- create the audio filter
afilter0 = sysctl0:audfilterCreate({name="afilter0", numOutputPorts=AUD_FILTER_OUTPUTS, numOutputFrames=32,
				      maxFramesSize=8192, filterType=AUD_FILTER_TYPE, cBufferLen=32});
afilter0:setParam({param=Q_AUDFLTR_CMP_MAX_INPUT_PORTS, value=AUD_FILTER_INPUTS});
afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FREQ, value=AUD_FILTER_OUT_FREQ});
afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FRAME_NUM_SAMPLES, value=AUD_FILTER_OUT_FRAMESIZE});
afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FREQ, value=AUD_FILTER_IN_FREQ});
afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=AUD_FILTER_NUM_CHAN});
afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=AUD_FILTER_IN_FRAMESIZE});

afilter0:activateCfg();

-- attach AV decoders to the audio filter
for i = 1, AUD_FILTER_INPUTS do
   iport = afilter0:addInputPort();
   afilter0:setInputPortSource({inport=iport, src=avdec[i], threshold=1});
end

-- create AVOC
avoc0 = sysctl0:avocCreate({name="avoc0"});

-- create AVSNs
avsn = {};
for i = 1, AUD_FILTER_OUTPUTS do
   avsn[i] = sysctl0:avsnCreate({name="avsn"..i});
   avsn[i]:bindAudioInput({src=afilter0, port=i-1});
end

-- add audio channels to AVOC   
ach = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
avoc0:setAudioChannelSource({ch=ach, src=avsn[OUTPUT_SEL]});
avoc0:enableAudioChannel({ch=ach, enable=1});


avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=OUTSAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
avoc0:activateCfg();

avoc0:activateAudioChannelCfg();

-- create stream readers
bsr = {};
rs0 = sysctl0:createReadStreamer({name="rs0"});
for i = 1, AUD_FILTER_INPUTS do
   rs0:bindDecoder({dec=avdec[i]});
   bsr[i] = rs0:open({type="qbox", file=INPUT_FILENAME[i], srcId={AUDIO_STREAM_ID[i]}});
end

avoc0:start();

for i = 1, AUD_FILTER_OUTPUTS do
   avsn[i]:start();
end

afilter0:start();

for i = 1, AUD_FILTER_INPUTS do
   avdec[i]:start();
   rs0:play({bsr=bsr[i]});
end

function q()
   rs0:query();
   avdec[1]:query();
   afilter0:query();
   avsn[1]:query();
   avoc0:query();
end
