--[[

This script is used to encode two D1 streams from 2 NTSC streams via svideo input.  The first
video port is used for display while both streams are streamed

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------
VOUT				= VOUT or "svideo"
FORMAT				= FORMAT or "480i"

-- Init arrays
ENCW 				= ENCW or {};
ENCH 				= ENCH or {};
BITRATE 			= BITRATE or {};
FRAMERATE 			= FRAMERATE or {};

-- Width and height of stream 1
ENCW[0] 			= ENCW[0] or 720;
ENCH[0] 			= ENCH[0] or 480;

-- Bitrate of stream 1
BITRATE[0] 			= BITRATE[0] or 2000000;

-- video input mode svideo or composite
--VIN="composite";
-- Changed code to accept composite and svideo both as inputs
VIN 				= VIN or {};
VIN[0]				= VIN[0] or "svideo";
VIN[1]				= VIN[1] or "composite";

-- Framerate of stream 2 (note 30fps interpreted as 29.97)
FRAMERATE[0] 		= FRAMERATE[0] or 30;

-- Width and height of stream 2
ENCW[1] 			= ENCW[1] or 720;
ENCH[1] 			= ENCH[1] or 480;

-- Bitrate of stream 2
BITRATE[1] 			= BITRATE[1] or 500000;

-- Framerate of stream 2 (note 30fps interpreted as 29.97)
FRAMERATE[1] 		= FRAMERATE[1] or 30;

-- Streaming config (port and port+1 will be used)
STREAMING 			= STREAMING or 1;
HOST 				= HOST or "192.168.40.38";
PORT 				= PORT or 9700;
MAXPACKETSIZE		= MAXPACKETSIZE or 768;

-- Enable or disable rate display
SHOWRATE 			= SHOWRATE or 0;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES 	= NUM_AVC_REFERENCES or 6;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- By default perform bitstream output
BITSTREAM_OUTPUT 	= 1;

-- Set starting output stream
OUTPUTCHANNEL 		= 0;

lvpp = {};
avcenc = {};
avsn = {};
vch =  {};
TICKSPERFRAME = {};

-- calculate duration of inputs
TICKSPERFRAME[0] = 3003 * 30 / FRAMERATE[0];
TICKSPERFRAME[1] = 3003 * 30 / FRAMERATE[1];

-- Set up output port
board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});

-- Create output
avoc0 = sysctl0:avocCreate({name="avoc0"});
avoc0:setOutput480i();

-- set both ports to live input
-- do this after creating AVOC when both will be live
sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});

-- Set up the output port
board.setVideoInput({port=0, format=FORMAT, mode=string.format("%s",VIN[0])});
board.setVideoInput({port=1, format=FORMAT, mode=string.format("%s",VIN[1])});

-- Create and start writestreamer
ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

for i = 0,1 do

  -- create video input
  lvpp[i] = sysctl0:lvppCreate({name="lvpp"..i, slot=0, port=i, maxWidth=720, maxHeight=480});

  -- Scale the inputs
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=TICKSPERFRAME[i]});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PICT_WIDTH,  value=ENCW[i]});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PICT_HEIGHT, value=ENCH[i]});
  lvpp[i]:setParam({param=Q_NVPP_CMP_PMEDS_H, value=2});
  lvpp[i]:setParam({param=Q_NVPP_CMP_PMEDS_V, value=2});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});

  -- create an avsn to allow output of the live channel
  avsn[i] = sysctl0:avsnCreate({name="avsn"..i});

  -- allocate a video channel
  vch[i] = avoc0:addVideoChannel();

  -- assign the channel to the vpp
  avoc0:setVideoChannelSource({ch=vch[i], src=avsn[i]});

  -- create encoder
  avcenc[i] = sysctl0:avcencCreate({name="avcenc"..i, ws=ws, maxwidth=ENCW[i], maxheight=ENCH[i], numreferenceframes=NUM_AVC_REFERENCES});

  -- start stream id's at 2
  videoStreamId = i + 2;

  -- configure for 480i
  avcenc_480i(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, 0);

  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=TICKSPERFRAME[i]});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});

  -- needed for streaming
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
  avcenc[i]:activateVideoEncCfg();

  -- connect to vpp
  avcenc[i]:bindVideoInput({src=lvpp[i]});

  -- connect avsn to vpp
  avsn[i]:bindVideoInput({src=lvpp[i]});

  avsn[i]:activateCfg();

  if STREAMING == 1 then
    ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", 
                HOST, PORT+i,MAXPACKETSIZE), sid = {videoStreamId}});
  end
  avcenc[i]:setStreamID({streamid=videoStreamId});

  -- ack infrequently to save cpu
  avcenc[i]:setAckInterval({interval=8});
end

-- activate the AVOC
avoc0:activateVideoChannelCfg();
avoc0:start();

-- function to switch between the inputs
function switch()
  print(string.format("Now display channel %d at bitrate %d\n", OUTPUTCHANNEL, BITRATE[OUTPUTCHANNEL]));
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BYPASS_CHANNEL, value=vch[OUTPUTCHANNEL]});
  avoc0:activateCfg();

  OUTPUTCHANNEL = OUTPUTCHANNEL + 1;
  if OUTPUTCHANNEL == 2 then
    OUTPUTCHANNEL = 0;
  end
end

-- init display
switch();

-- start video pipeline
for i = 0,1 do
  avcenc[i]:record();
  avsn[i]:start();
  lvpp[i]:start();
end

