-- MUST RUN THE FOLLOWING OUTSIDE LUA TO SET UP THE BOARD
-- /home/merlin/bin/loadfpga.sh /home/merlin/test/merlinbup/kestrel/merlinbup_fpga.bin

NUMCH 		= NUMCH or 8;
OUTCH 		= OUTCH or 8;
OUTW 		= OUTW or 320;
OUTH 		= OUTH or 240;
COMPWIDTH 	= COMPWIDTH or 1280;
COMPHEIGHT 	= COMPHEIGHT or 480;
VIDEO_FORMAT = VIDEO_FORMAT or "720p"
VOUT	 	= VOUT or "hdmi"

-- Set up the input port
sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00048000});
sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000});

-- Set up 54MHz VOUT clock
sysctl0:configure({param=Q_SYS_CFG_V0_PIXEL_CLOCK_FLAGS, value=(Q_SYS_CFP_CLOCK_FLAG_OUTPUT + Q_SYS_CFP_CLOCK_FLAG_INPUT)});
sysctl0:configure({param=Q_SYS_CFG_V0_PIXEL_CLOCK_HZ, value=54000000});
sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_1, value=0x000026c9});

-- Set up techwell part
os.execute("sh /home/merlin/test/merlinbup/kestrel/set_2835_ntsc.i2c");

-- Set up the output port
board.setVideoOutput({port=0, format=VIDEO_FORMAT, mode=VOUT});

-- fatal errors only
sysctl0:setVerboseLevel({param=1});

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x80101080, interleave=NUMCH, syncmode=Q_SYS_CFP_VPORT_SYNC_MODE_MOBI});
sysctl0:setVPortMode({port=1, live=0});

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput720p60();
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=1280});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=720});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_RATE_DIVIDER, value=2});
avoc0:activateCfg();

NUMCH = NUMCH - 1;
OUTCH = OUTCH - 1;
lvpp = {};
avsn = {};

for i = 0,NUMCH do
  lvpp[i] = sysctl0:lvppCreate({name="lvpp0", slot=i, port=0, maxWidth=720, maxHeight=480, maxPMEDSFactor=2, numVidFrame=8});
  avsn[i] = sysctl0:avsnCreate({name="avsn0"});
end

for i = 0,NUMCH do
  lvpp[i]:crop({cropWidth=720, cropHeight=480, vertOffset=0, horzOffset=0});
  lvpp[i]:size({outputWidth=720, outputHeight=480});
  lvpp[i]:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=262});
  lvpp[i]:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp[i]:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
  lvpp[i]:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
  lvpp[i]:setParam({param=Q_LVPP_CMP_FAST_SCALE_MODE, value=1});
  lvpp[i]:activateCfg();
end

for i = 0,OUTCH do
  avsn[i]:bindVideoInput({src=lvpp[i]});
end

vch = {};
x = 0;
y = 0;
for i = 0,OUTCH do
  vch[i] = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch[i], src=avsn[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=720});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=480});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

avoc0:start();

for i = 0,OUTCH do
  avsn[i]:start();
end

for i = 0,NUMCH do
  lvpp[i]:start();
end
