--[[

This script is used to encode from live video input.  
The reconstructed preview is displayed on the screen. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------
-- Video input mode
-- "hdmi" HDMI input and output
-- "component"
-- "svideo" 
-- "composite" 
VIN             = VIN or "component";

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "component" Component output
-- "svideo" SVIDEO output (note composite is driven at same time on output)
VOUT            = VOUT or "hdmi";

-- Video format for capture and display.
-- "1080i"
-- "1080i50"
-- "720p"
-- "720p50"
-- "480i"
-- "576i"
FORMAT          = FORMAT or "1080i";

-- Video bitrate
if (FORMAT == "720p")  or (FORMAT == "720p50") then
  VBITRATE 			= VBITRATE 			or 6000000
elseif (FORMAT == "1080i")  or (FORMAT == "1080i50") then
  VBITRATE 			= VBITRATE 			or 6000000
elseif (FORMAT == "480i") or (FORMAT == "576i") then
  VBITRATE 			= VBITRATE 			or 2000000
end

SHOWRATE = SHOWRATE or 0;

--Video quality override
--This option is to override the quality constraints that would kick in
--if the user parameters does not meet quality expectations.
QUALITY_OVERRIDE = QUALITY_OVERRIDE or 0;

-- Allow video only encode
VIDEO_ONLY = VIDEO_ONLY or 0

--Generate 1264 compatible stream
COMPATIBLE1264 = COMPATIBLE1264 or 0

-- Debug variable to disable bitstream output to host
BITSTREAM_OUTPUT =  BITSTREAM_OUTPUT or 1;

-- Controls the audio encoder type
--   "AAC-LC"
--   "ER-AAC-LD"
--   "MPEG-LAYER-2"
--   "G711-A-LAW"
--   "G711-MU-LAW" 
--   "G711" (defaults to mu-law) 
--   "G722"
--   "G726"
--   "G728"
--   "PCM"
AUDIO_ENCODER = AUDIO_ENCODER or "AAC-LC";

-- Controls the audio input type
--   "onboard" for external port connected to ADC 
--   "hdmi" for HDMI audio-in
AIN = AIN or "onboard";

if (AIN == "onboard") then
  AIN_PORT = AIN_PORT or 0;
  -- Input audio sample rate (may be overridden depending on codec)
  SAMPLERATE = SAMPLERATE or 32000;
else
  AIN_PORT = 1;
  -- To do: The sample rate should be obtained by reading a register of
  -- HDMI input chip instead of hard coding here
  SAMPLERATE = SAMPLERATE or 48000;
end

-- Audio stereo/mono (may be overridden depending on codec)
AUDIO_STEREO = AUDIO_STEREO or 1;

-- Audio bitrate (may be overridden depending on codec)
if(AUDIO_STEREO) then
   ABITRATE = ABITRATE or 64000;
else
   ABITRATE = ABITRATE or 56000;
end

-- Audio frame size (may be overridden depending on codec)
AUDIO_FRAME_SIZE = AUDIO_FRAME_SIZE or 256; 

-- Controls the GOP structure
-- "IBBRBP" : Pyramidal GOP structure
-- "IP" : No B frames
GOP_STRUCTURE =  GOP_STRUCTURE or "IBBRBP";

-- Controls the coding structure of a frame or field pair
-- "mbaff" : mb adaptive field/frame
-- "field" : field coding
-- "frame" : frame coding.
-- Note progressive inputs are forced to frame coding
FRAME_STRUCTURE =  FRAME_STRUCTURE or "mbaff";

-- Controls the profile of the AVC bitstream
-- "baseline" : forces IP GOP structure and frame coding, no CABAC
-- "main"     : B frames, field/MBAFF coding and CABAC allowed
-- "high"     : all encoding options available
PROFILE = PROFILE or "high";

if (PROFILE == "baseline") then
  GOP_STRUCTURE = "IP";
  FRAME_STRUCTURE = "frame";
end

--output file(s)
OUTFILE_H4V = OUTFILE_H4V or nil; --"test.h4v"
OUTFILE_MP4 = OUTFILE_MP4 or nil; --"test.mp4"
OUTFILE_MKV = OUTFILE_MKV or nil; --"test.mkv"
OUTFILE_ADTS = OUTFILE_ADTS or nil; --"test.aac"
OUTFILE_QBOX = OUTFILE_QBOX or nil; --"test.qbx"
OUTFILE_TS   = OUTFILE_TS or nil; --"test.ts"

--Circular buffer size for last few seconds of QBOX recording in RAM
CIRCULAR_SIZE = CIRCULAR_SIZE or 0;      --Make it non-zero to enable circular writer
                                         --The size should be selected based on available host memory

--Buffer size used by circular writer for post-trigger recording
POST_SIZE = POST_SIZE or 0;       --Make it non-zero to enable post trigger
                                  --recording from circular writer

--The above buffers would be saved to QBOX or TS files depending on  
--STREAM_FORMAT using function saveprepostbuf in this script. In case of QBOX,
--they would also be converted to mp4 or mkv format using the streamexport utility.
-- The output format for streamexport will be defined using the following:
STREAMEXPORT_FORMAT = STREAMEXPORT_FORMAT or "mkv"  -- "mkv" or "mp4"
                                                    -- "mp4" will not have audio

--option to encode at half framerate (to get 720p30 from 720p60 input, etc.)
HALF_FRAMERATE = HALF_FRAMERATE or 0;

--option to turn on inverse telecine
INVERSE_TELECINE = INVERSE_TELECINE or 0;

-- Controls live vs reconstructed preview
-- 0 : no preview
-- 1 : live
-- 2 : reconstructed
PREVIEW =  PREVIEW or 1;

-- Controls network streaming configuration
-- 0: no streaming
-- 1: udp 
-- 2: rtp
STREAMING = STREAMING or 0;

-- UDP Settings
HOST = HOST or "192.168.40.120";
PORT = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;
-- Set to 0 for streaming elementary H264 to PC client (like VLC)
-- Set to 1 for streaming QBOX to another board
-- Not used for TS streams
BOARD_TO_BOARD = BOARD_TO_BOARD or 0;

-- Uses IP-only bitstreams for streaming via UPD or RTP by default
-- If B frames are needed, just modify this part
if (STREAMING > 0) then
   GOP_STRUCTURE = "IP";
end
                                         
-- RTP/RTSP Settings
-- ip address of rtp server.
SERVER_IP = SERVER_IP or "NULL";
-- port of rtp server
SERVER_PORT = SERVER_PORT or 554;
-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test";
-- Session Timeout Interval in Seconds
SESSION_TIMEOUT = SESSION_TIMEOUT or 60;
-- rtp server mode
SERVER_MODE = SERVER_MODE or 0;
-- Maximum number of allowed client connections
MAX_CLIENTS_ALLOWED = MAX_CLIENTS_ALLOWED or 100;	--Putting a large number so that we do not impose any limit.
                                                        --For testing, a small number can be specified.

--User name for client authentication
USER_NAME = USER_NAME or "NULL";
--Password for client authentication
PASSWORD = PASSWORD or "NULL";

-- Multicast IP, make it NULL for unicast
-- For the multi-cast address, you can use any
-- class D IP addresses (224.0.0.0 to 239.255.255.255), but out of
-- this range some are reserved for special uses which we should not
-- use, so it is better to use unassigned range (225.0.0.0 to 238.255.255.255)
MULTICAST_IP = MULTICAST_IP or "NULL";
-- Multicast TS Port
MULTICAST_TS_PORT = MULTICAST_TS_PORT or 4570;
-- Multicast Video Port
MULTICAST_VID_PORT = MULTICAST_VID_PORT or 4567;
-- Multicast Audio Port
MULTICAST_AUD_PORT = MULTICAST_AUD_PORT or 1234;

-- Set 1 to create third party client
STREAM_TO_THIRD_PARTY_CLIENT = STREAM_TO_THIRD_PARTY_CLIENT or 0;

-- Third party client Ip address
THIRD_PARTY_CLIENT_IP = THIRD_PARTY_CLIENT_IP or "10.32.84.197"

-- Third party client Port
THIRD_PARTY_CLIENT_PORT = THIRD_PARTY_CLIENT_PORT or 2048;

-- HTTP Port number
HTTP_PORT = HTTP_PORT or 8080;

-- RTCP Session Enable Flag
RTCP_FLAG = RTCP_FLAG or 0;

-- RTCP Sender Report Interval in milliseconds
RTCP_SR_INTERVAL = RTCP_SR_INTERVAL or 5000;

FILE_STREAMER       = FILE_STREAMER or 0;
FILE_STREAMER_MOUNT_DIR = "/directmnt/qa/streams/merlin/filestreamer/"

-- Enable Analytics Streaming
-- 0 : Don't stream analytics
-- 1 : Stream analytics as metadata
-- 2 : Stream analytics in Private SEI
ENABLE_ANALYTICS_STREAMING = ENABLE_ANALYTICS_STREAMING or 0

-- Enable monitoring of Client's tcp send buffer fullness
-- 0 - Do not monitor 
-- n - Discard sending frames to TCP client's 
--	   when send buffer is getting filled more than 
--	   n percentage of max socket's send buffer
SEND_BUFFER_FULLNESS_LIMIT = SEND_BUFFER_FULLNESS_LIMIT or 0

-- End of RTP/RTSP Settings

-- Enable to burn in timecode for either 8x8, 16x16 or 32x32
BURNIN_TIMECODE = BURNIN_TIMECODE or 0;

-- Enable burn-in at the LVPP object
-- 0: Burn-in done at AVC encoder
-- 1: Burn-in done at LVPP
BURNIN_AT_LVPP = BURNIN_AT_LVPP or 0;

-- 8, 16 or 32
BURNIN_FONTSIZE = BURNIN_FONTSIZE or 16;

BURNIN_FONT = {};
BURNIN_FONT[8] = "/var/mobiapp/fonts/MGSysFont08.bin";
BURNIN_FONT[16] = "/var/mobiapp/fonts/MGSysFont16.bin";
BURNIN_FONT[32] = "/var/mobiapp/fonts/MGSysFont32.bin";

-- Deinterlacing can be enabled for 480i/576i modes.  Reduces performance.
-- Use frame coding when deinterlacing is on.
-- 0 to disable, 3 to deinterlace
DEINT_MODE = DEINT_MODE or 0;
if (DEINT_MODE > 0 ) then
  FRAME_STRUCTURE = "frame";
end

-- Use high quality encoding.  For 480i only.
HIGHQUALITY = 1;

-- Segment Encoding Mode. 0 or 1 for disable. 2 for 2 segments per picture, 3 for 3 segments per picture 4 for 4 segments per picture ...
SEGMENT_ENCODE_MODE = SEGMENT_ENCODE_MODE or 0;

-- Parameter to enable OSD. Set 1 to enable OSD , 0 to disable OSD.
OSD_ENABLE 			= OSD_ENABLE or 0;

OSD_MEMORY_SIZE     = OSD_MEMORY_SIZE or (4*1024*1024);

-- 0: QBOX
-- 2: TS
STREAM_FORMAT = STREAM_FORMAT or 0;    

TS_PACK_NUMBER = TS_PACK_NUMBER or 0; 

-- Enable Preprocessing delay. This must be enabled to make motion detection work properly
-- with other preprocessing features (privacy region, burn-in, etc).
-- 1: Enable. 0: Disable.
PREPROC_DELAY_ENABLE = PREPROC_DELAY_ENABLE or 0;
 
-- Set motion detection mode
-- 0 = off
-- 1 = enabled at LVPP
-- 2 = enabled at AVC
MOTIONDETECTION = 0;

-- Enable Camera Tamper
CAMERATAMPER    = CAMERATAMPER    or 0

-- Enable motion detection if analytics streaming is enabled 
if (ENABLE_ANALYTICS_STREAMING > 0) then
    MOTIONDETECTION = 2;
end 

-- Enable adaptive framerate/bitrate when motion is present.  Note that
-- MOTIONDETECTION must be enabled.
-- 50kbps/3fps no motion, 2mbits/30fps with motion as example here
ADAPTIVE_BITRATE_MOTIONDETECTION_ENABLE = ADAPTIVE_BITRATE_MOTIONDETECTION_ENABLE or 0;
ADAPTIVE_BITRATE_MOTIONDETECTION_LO_RATE = ADAPTIVE_BITRATE_MOTIONDETECTION_LO_RATE or 50000;
ADAPTIVE_BITRATE_MOTIONDETECTION_HI_RATE = ADAPTIVE_BITRATE_MOTIONDETECTION_HI_RATE or 2000000;
ADAPTIVE_BITRATE_MOTIONDETECTION_LO_FRAMETICKS = ADAPTIVE_BITRATE_MOTIONDETECTION_LO_FRAMETICKS or 30030;
ADAPTIVE_BITRATE_MOTIONDETECTION_HI_FRAMETICKS = ADAPTIVE_BITRATE_MOTIONDETECTION_HI_FRAMETICKS or 3003;
ADAPTIVE_BITRATE_MOTIONDETECTION_DURATION = ADAPTIVE_BITRATE_MOTIONDETECTION_DURATION or 2;

--[[ Audio filter setup ]]--
-- Enable audio filter. Insert the filter between the SAIN and Audio Enc objects.
AUD_FILTER_ENABLE = AUD_FILTER_ENABLE or 0;
-- Number of input ports of the audio filter
AUD_FILTER_INPUTS = AUD_FILTER_INPUTS or 1;
-- Number of output ports of the audio filter
AUD_FILTER_OUTPUTS = AUD_FILTER_OUTPUTS or 1;
-- Audio filter type
-- Q_AUDFLTR_CFP_TYPE_ALLPASS:            An all-pass filter
-- Q_AUDFLTR_CFP_TYPE_RATECONVERSION:     Audio resampling filter
AUD_FILTER_TYPE = AUD_FILTER_TYPE or Q_AUDFLTR_CFP_TYPE_RATECONVERSION;

-- Audio encoder's sample rate. Can be different than the input sample rate if
-- using the audio resampling filter. (May be overridden depending on codec)
ENC_SAMPLERATE = ENC_SAMPLERATE or 8000;

-- Privacy regions setup
-- 0: The privacy region will be processed at the AVC encoder layer
-- 1: The privacy region will be processed at the LVPP layer
PRIV_REGION_AT_LVPP = PRIV_REGION_AT_LVPP or 0;

-- Variable frame rate with NTP clock timestamp
VARIABLE_FRAME_RATE_WITH_NTP = VARIABLE_FRAME_RATE_WITH_NTP or 0
-- Controls the frequency of NTP updates to codec.
-- Unit in seconds.
NTP_UPDATE_INTERVAL = NTP_UPDATE_INTERVAL or 30;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- Set RTP_AUDIO 
if (VIDEO_ONLY == 1) then
  RTP_AUDIO = "none";
elseif (AUDIO_ENCODER == "AAC-LC") then
  RTP_AUDIO = "aac";
elseif (AUDIO_ENCODER == "PCM") then
  RTP_AUDIO = "pcm";
elseif (AUDIO_ENCODER == "G711-A-LAW") and (STREAM_FORMAT ~= 2) then
  RTP_AUDIO = "g711a";
elseif ((AUDIO_ENCODER == "G711-MU-LAW") or (AUDIO_ENCODER == "G711")) and (STREAM_FORMAT ~= 2) then
  RTP_AUDIO = "g711u";
elseif (AUDIO_ENCODER == "G726") and (STREAM_FORMAT ~= 2) then
  RTP_AUDIO = "g726";
elseif (AUDIO_ENCODER == "MPEG-LAYER-2") and (STREAM_FORMAT ~= 2) then
  RTP_AUDIO = "mpa";
else
  RTP_AUDIO = "none";
end

if (STREAM_FORMAT ~= 2) then
  RTP_MP2T = 0;
else
  RTP_MP2T = 1;
end

PTS_120KHZ = PTS_120KHZ or 0;

NUM_OF_INTERVAL_LINES = 0;
if(SEGMENT_ENCODE_MODE ~= 0) and (SEGMENT_ENCODE_MODE ~= 1) then


  if (FORMAT == "720p") or (FORMAT == "720p50") then
      if(SEGMENT_ENCODE_MODE ==  2) then
          NUM_OF_INTERVAL_LINES = 368;
      elseif(SEGMENT_ENCODE_MODE ==  3) then
          NUM_OF_INTERVAL_LINES = 256;
      elseif(SEGMENT_ENCODE_MODE ==  4) then
          NUM_OF_INTERVAL_LINES = 192;
      else 
          print("Warning, Not supported !");
      end
  elseif (FORMAT == "1080i") or (FORMAT == "1080i50") then
      if(SEGMENT_ENCODE_MODE ==  2) then
          NUM_OF_INTERVAL_LINES = 272;
      elseif(SEGMENT_ENCODE_MODE ==  3) then
          NUM_OF_INTERVAL_LINES = 192;
      elseif(SEGMENT_ENCODE_MODE ==  4) then
          NUM_OF_INTERVAL_LINES = 144;
      else 
          print("Warning,Not supported !");
      end
  elseif (FORMAT == "480i") then
      if(SEGMENT_ENCODE_MODE ==  2) then
          NUM_OF_INTERVAL_LINES = 128;
      elseif(SEGMENT_ENCODE_MODE ==  3) then
          NUM_OF_INTERVAL_LINES = 96;
      elseif(SEGMENT_ENCODE_MODE ==  4) then
          NUM_OF_INTERVAL_LINES = 64;
      else 
          print("Warning,Not supported !");
      end
  end

  
 
  -- Segment encoding only works with IP sequence, so let's override the GOP structure
  if(GOP_STRUCTURE ~= "IP") then
    GOP_STRUCTURE = "IP";
    print("Warning, Segment Encoding mode is active, GOP structure has been overridden");
  end
   
  -- For interlaced encoding, segment encoding only works with "field" mode
  if(FORMAT == "1080i") or (FORMAT == "480i") then
    if( FRAME_STRUCTURE ~= "field") then
      FRAME_STRUCTURE = "field";
      print("Warning, Segment Encoding mode is active, frame structure has been overridden");
    end
  end
  -- Segment encoding only works without preview, override if needed
  if(PREVIEW ~= 0) then
    PREVIEW = 0;
    print("Warning, Segment Encoding mode is active, preview mode has been turned off!");
  end
end

ENC_OUTPUT_BUFFER = VBITRATE / 2;
ENC_GUARD_BAND_PER_MB = 200;

if (FORMAT == "720p") then
  HSIZE         = 1280;  
  VSIZE         = 720;
  FRAME_STRUCTURE = "frame"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  if (PTS_120KHZ == 1) then
    VIN_STC = 2002;
  else
    VIN_STC = 1500;
  end
  VIN_OUTPUT_STC = VIN_STC;
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 3;
  else
    NUM_VIN_FRAMES = 10;
    NUM_AVC_REFERENCES = 10;
    AVSN_DELAY = 8;
  end
  AVC_LEVEL = 40;
  if (VIN == "component") then
       		 HSTART_PIXEL = 291;
  else
        	HSTART_PIXEL = 251;
  end
  if (ENC_OUTPUT_BUFFER < 2000000) then
    ENC_OUTPUT_BUFFER = 2000000;
  end
elseif (FORMAT == "720p50") then
  HSIZE         = 1280;  
  VSIZE         = 720;
  FRAME_STRUCTURE = "frame"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 3;
  else
    NUM_VIN_FRAMES = 10;
    NUM_AVC_REFERENCES = 10;
    AVSN_DELAY = 8;
  end

  if (PTS_120KHZ == 1) then
    VIN_STC = 2400;
  else
    VIN_STC = 1800;
  end
  VIN_OUTPUT_STC = VIN_STC;
  
  AVC_LEVEL = 40;
  if (VIN == "component") then
   HSTART_PIXEL = 291;
   else
  HSTART_PIXEL = 251;
  end
  if (ENC_OUTPUT_BUFFER < 2000000) then
    ENC_OUTPUT_BUFFER = 2000000;
  end
elseif (FORMAT == "1080i") then
  HSIZE = 1920;  
  VSIZE = 1080;
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  if (PTS_120KHZ == 1) then
    VIN_STC = 4004;
  else
    VIN_STC = 3003;
  end
  VIN_OUTPUT_STC = VIN_STC;
  ENC_OUTPUT_BUFFER = ENC_OUTPUT_BUFFER + VBITRATE / 4;
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 6;
    AVSN_DELAY = 2;
  else
    NUM_VIN_FRAMES = 11;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 8;
  end
  AVC_LEVEL = 40;
   if (VIN == "component") then
        HSTART_PIXEL = 235;
  else
        HSTART_PIXEL = 193;
  end
  if (ENC_OUTPUT_BUFFER < 4000000) then
    ENC_OUTPUT_BUFFER = 4000000;
  end
elseif (FORMAT == "1080i50") then
  HSIZE = 1920;  
  VSIZE = 1080;
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  if (PTS_120KHZ == 1) then
    VIN_STC = 4800;
  else
    VIN_STC = 3600;
  end
  VIN_OUTPUT_STC = VIN_STC;
  ENC_OUTPUT_BUFFER = ENC_OUTPUT_BUFFER + VBITRATE / 4;
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 6;
    AVSN_DELAY = 2;
  else
    NUM_VIN_FRAMES = 11;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 8;
  end
  AVC_LEVEL = 40;
   if (VIN == "component") then
        HSTART_PIXEL = 235;
  else
        HSTART_PIXEL = 193;
  end
  if (ENC_OUTPUT_BUFFER < 4000000) then
    ENC_OUTPUT_BUFFER = 4000000;
  end
elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 9;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 0;
  NUM_AVC_REFERENCES = 8;
  if (PTS_120KHZ == 1) then
    VIN_STC = 4004;
  else
    VIN_STC = 3003;
  end
  VIN_OUTPUT_STC = VIN_STC;
  if (GOP_STRUCTURE == "IP") then
    AVSN_DELAY = 3;
  else
    AVSN_DELAY = 4;
  end
  AVC_LEVEL = 31;
  if (FRAME_FORMAT == "frame") then
    AVC_LEVEL = 30;
  end
  if (ENC_OUTPUT_BUFFER < 1000000) then
    ENC_OUTPUT_BUFFER = 1000000;
  end
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 10;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 0;
  NUM_AVC_REFERENCES = 8;
  if (PTS_120KHZ == 1) then
    VIN_STC = 4800;
  else
    VIN_STC = 3600;
  end
  VIN_OUTPUT_STC = VIN_STC;
  if (GOP_STRUCTURE == "IP") then
    AVSN_DELAY = 4;
  else
    AVSN_DELAY = 4;
  end
  AVC_LEVEL = 31;
  if (FRAME_FORMAT == "frame") then
    AVC_LEVEL = 30;
  end
  if (ENC_OUTPUT_BUFFER < 1000000) then
    ENC_OUTPUT_BUFFER = 1000000;
  end
else
  error("Invalid format " .. tostring(FORMAT))
end

-- Sanity check: raise the level if bitrate is too high for it
if ((VBITRATE > 135000000) and (AVC_LEVEL < 51)) then
  AVC_LEVEL = 51;
elseif ((VBITRATE > 50000000) and (AVC_LEVEL < 50)) then
  AVC_LEVEL = 50;
elseif ((VBITRATE > 20000000) and (AVC_LEVEL < 41)) then
  AVC_LEVEL = 41;
elseif ((VBITRATE > 14000000) and (AVC_LEVEL < 40)) then
  AVC_LEVEL = 40;
elseif ((VBITRATE > 10000000) and (AVC_LEVEL < 31)) then
  AVC_LEVEL = 31;
elseif ((VBITRATE > 4000000) and (AVC_LEVEL < 30)) then
  AVC_LEVEL = 30;
end

-- Sanity check: cap output buffer size so we don't run out of memory
-- even if we have enough memory the ENC_OUTPUT_BUFFER cann't exceed 16M for hardware limits 
if (ENC_OUTPUT_BUFFER > 15000000) then
  ENC_OUTPUT_BUFFER = 15000000;
end


-- Deinterlacing defaults to producing 60 fps output. The firmware will calculate the right values
-- but the values must be a multiple of 2 so it divides properly.
if (DEINT_MODE ~= 0) then
  if (PTS_120KHZ == 1) then
    if FORMAT == "576i" then
      VIN_STC = 4800
    else
      VIN_STC = 4004
    end
  else
    if FORMAT == "576i" then
      VIN_STC = 3600
    else
      VIN_STC = 3003
    end
  end
  VIN_OUTPUT_STC = VIN_STC;
end

-- Framerate is reduced by increasing the length (in ticks) of the frames when sent from the
-- video input module.  In this case, the output length is set to 2x normal so the output
-- frequency is 1/2 normal.
if (HALF_FRAMERATE ~= 0) then
  VIN_OUTPUT_STC = VIN_OUTPUT_STC * 2;
end
-- Timing for reconstructed preview is affected by framerate and the presence of audio
if ((HALF_FRAMERATE ~= 0) or (VIDEO_ONLY ~= 0)) then
  AVSN_DELAY = AVSN_DELAY - 1;
end

if (GOP_STRUCTURE == "IP") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_IP;
elseif  (GOP_STRUCTURE == "IBBRBP") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
elseif  (GOP_STRUCTURE == "IBP") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_IBP;
elseif  (GOP_STRUCTURE == "I") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_I;
else
  print("Invalid GOP structure " .. GOP_STRUCTURE .. "\n");
end

-- modulo 16 size
VSIZE16 = 16*math.ceil(VSIZE/16)

VIDEOSTREAMID = 2;
AUDIOSTREAMID = 1;

-- Function to start recording
function record()
  -- add bitstream writers
  if (OUTFILE_H4V) then
    ws:open({type="h4v", file=OUTFILE_H4V, sid = {VIDEOSTREAMID}});
  end
  if (OUTFILE_MP4) then
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_GOP_SIZE, value=16});
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_QMED_ENABLE, value=1});
  
    ws:open({type="mp4", file=OUTFILE_MP4, sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
  end
  if (OUTFILE_MKV) then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_GOP_SIZE, value=16});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_QMED_ENABLE, value=1});

    ws:open({type="mkv", file=OUTFILE_MKV, sid = {VIDEOSTREAMID,AUDIOSTREAMID}});
  end


  if (OUTFILE_ADTS) then
    ws:open({type="adts", file=OUTFILE_ADTS, sid = {AUDIOSTREAMID}});
  end
  if (STREAM_FORMAT ~= 2) then 
    if (OUTFILE_QBOX) then
      ws:open({type="file", file=OUTFILE_QBOX, sid = {0}});
    end
  else
    if (OUTFILE_TS) then
      ws:open({type="file", file=OUTFILE_TS, sid = {0}});
    end
  end
  if (CIRCULAR_SIZE > 0) then
    print("Opening a circular bitstream writer");
    if (VIDEO_ONLY == 1) then
      bswc=ws:open({type="circular", file=string.format("prebufferlen=%d,postbufferlen=%d,streamformat=%d", 
          CIRCULAR_SIZE, POST_SIZE, STREAM_FORMAT), sid = {VIDEOSTREAMID}});
    else
      bswc=ws:open({type="circular", file=string.format("prebufferlen=%d,postbufferlen=%d,streamformat=%d", CIRCULAR_SIZE, POST_SIZE, STREAM_FORMAT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    end
  end


  if (STREAMING == 1) then
    if (BOARD_TO_BOARD == 0 and STREAM_FORMAT ~=2) then
      ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", HOST, PORT, MAX_PACKET_SIZE), sid = {VIDEOSTREAMID}});
    elseif (VIDEO_ONLY == 1) then
      ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=%d", HOST, PORT, MAX_PACKET_SIZE), sid = {VIDEOSTREAMID}});
    else
      ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=%d", HOST, PORT, MAX_PACKET_SIZE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    end
  elseif(STREAMING == 2) then
      if (VIDEO_ONLY == 0 and RTP_AUDIO == "none") then
        if(RTP_MP2T == 0) then
          print("Audio encoder " .. AUDIO_ENCODER .. " not supported in RTP so just streaming video\n");
         else
           print("Audio encoder " .. AUDIO_ENCODER .. " not supported in TS so just streaming video\n");
         end
      end

      if(STREAM_TO_THIRD_PARTY_CLIENT == 1) then
        if (RTP_AUDIO == "none") then
          ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=none,mod=%d,msz=1472,mcli=%d,mp2t=%s,sbl=%d,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, MAX_CLIENTS_ALLOWED, RTP_MP2T, SEND_BUFFER_FULLNESS_LIMIT, THIRD_PARTY_CLIENT_IP,THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID}});
        else
          ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=%s,mod=%d,msz=1472,mcli=%d,mp2t=%s,sbl=%d,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, RTP_AUDIO, SERVER_MODE, MAX_CLIENTS_ALLOWED, RTP_MP2T, SEND_BUFFER_FULLNESS_LIMIT, THIRD_PARTY_CLIENT_IP,THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
        end
      else
        if (RTP_AUDIO == "none") then
          ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=none,mod=%d,msz=1472,mcli=%d,mp2t=%s,sbl=%d,mip=%s,mpts=%d,mptv=%d,htp=%d,aim=%d,rtcp=%d,sri=%d,sto=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, MAX_CLIENTS_ALLOWED, RTP_MP2T, SEND_BUFFER_FULLNESS_LIMIT, MULTICAST_IP, MULTICAST_TS_PORT, MULTICAST_VID_PORT, HTTP_PORT, ENABLE_ANALYTICS_STREAMING,RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT,USER_NAME,PASSWORD), sid = {VIDEOSTREAMID}});
        else
          ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=%s,mod=%d,msz=1472,mcli=%d,mp2t=%s,sbl=%d,mip=%s,mpts=%d,mptv=%d,mpta=%d,htp=%d,aim=%d,rtcp=%d,sri=%d,sto=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID, RTP_AUDIO, SERVER_MODE, MAX_CLIENTS_ALLOWED, RTP_MP2T, SEND_BUFFER_FULLNESS_LIMIT, MULTICAST_IP, MULTICAST_TS_PORT, MULTICAST_VID_PORT, MULTICAST_AUD_PORT, HTTP_PORT, ENABLE_ANALYTICS_STREAMING,RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT,USER_NAME,PASSWORD), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
        end
      end
  end
  -- start output
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    avsn0:start();
  end
  -- start encoders
  if (VIDEO_ONLY == 0) then
    audenc0:record();
  end
  avcenc0:record();
  -- start audio/video input
  lvpp0:start();
  if (VIDEO_ONLY == 0) then
     if (AUD_FILTER_ENABLE == 1) then
	audfil0:start(); -- starts audio filter before SAIN does
     end
    sain0:start();
  end
end

-- Function to stop recording
function stop()
    lvpp0:flush();
    -- Following flush is not needed as lvpp flush will flush the downstream 
    -- components
    --    avcenc0:flush();    -- This should close the write streamer
    
    if (VIDEO_ONLY == 0) then
        sain0:flush();
        -- Following is not needed as as sain0:flush operation sends a flush message
        -- that's passed on from one component to the next, stopping them all in order
        -- audenc0:stop();
    end
    
    avcenc0:waitForLastBlock();
    
    if (VIDEO_ONLY == 0) then
       audenc0:waitForLastBlock();
    end
    
    avcenc0:stop();
    avsn0:stop();
end

-- Function to setup parameters for a privacy region for AVC encoder. 
-- It can start a new region or move/resize an existing region.
-- obj: pointer to an AVC encoder object
-- i  : region index. valid values from 0 to 15 for a total of 16 regions
-- x  : X coordinate of region; unit is MB (macroblocks)
-- y  : Y coordinate of region; unit is MB
-- w  : Width of region; unit is MB
-- h  : Height of region; unit is MB
function setAVCPrivacyRegion(obj, i, x, y, w, h)
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_ENABLE, value=1});
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_MB_XPOS, value=x});
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_MB_YPOS, value=y});
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_MB_WIDTH, value=w});
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_MB_HEIGHT, value=h});
   obj:activateVideoEncCfg();
end

-- Function to remove a privacy region for AVC encoder
-- obj: pointer to an AVC encoder object
-- i  : region index. valid values from 0 to 15 for a total of 16 regions
function removeAVCPrivacyRegion(obj, i)
   obj:setVideoEncPrivacyParam({idx=i, param=Q_AVCENC_CMP_PREGION_ENABLE, value=0});
   obj:activateVideoEncCfg();
end

-- Function to setup parameters for a privacy region for LVPP.
-- It can start a new region or move/resize an existing region.
-- obj: pointer to a LVPP object
-- i  : region index. valid values from 0 to 15 for a total of 16 regions
-- x  : X coordinate of region; unit is MB (macroblocks)
-- y  : Y coordinate of region; unit is MB
-- w  : Width of region; unit is MB
-- h  : Height of region; unit is MB
function setLVPPPrivacyRegion(obj, i, x, y, w, h)
   if (PRIV_REGION_AT_LVPP == 1) then
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_ENABLE, value=1});
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_MB_XPOS, value=x});
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_MB_YPOS, value=y});
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_MB_WIDTH, value=w});
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_MB_HEIGHT, value=h});
      obj:activateCfg();
   else
      print("Privacy region function for LVPP is not enabled!\n");
   end
end

-- Function to remove a privacy region for LVPP
-- obj: pointer to a LVPP object
-- i  : region index. valid values from 0 to 15 for a total of 16 regions
function removeLVPPPrivacyRegion(obj, i)
   if (PRIV_REGION_AT_LVPP == 1) then
      obj:setPrivacyParam({index=i, param=Q_LVPP_CMP_PRIVACY_REGION_ENABLE, value=0});
      obj:activateCfg();
   else
      print("Privacy region function for LVPP is not enabled!\n");
   end
end

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

local VPORT_LIVE_MODE = 1;
if (NUM_OF_INTERVAL_LINES ~= 0) then
  -- if sequence encoding, we need to set VPortMode to live = 2 for lvpp to generate line interrupts
  VPORT_LIVE_MODE = 2;
end

if(OSD_ENABLE == 1) then
    -- OSD_BUFFER_WIDTH is the physical width of the buffer in pixels which is
    -- typically 2048/bytes per pixel
	OSD_X_OFFSET		= OSD_X_OFFSET or 0;
	OSD_Y_OFFSET		= OSD_Y_OFFSET or 0;
	if(FORMAT == "1080i") or (FORMAT == "1080i50") then
		OSD_DISPLAY_WIDTH  	= OSD_DISPLAY_WIDTH or 1024;
		OSD_DISPLAY_HEIGHT 	= OSD_DISPLAY_HEIGHT or 1080;
	elseif(FORMAT == "720p") or (FORMAT == "720p50") then
		OSD_DISPLAY_WIDTH  	= OSD_DISPLAY_WIDTH or 640;
		OSD_DISPLAY_HEIGHT 	= OSD_DISPLAY_HEIGHT or 720;
	elseif(FORMAT == "480i") then
        -- we can do full screen 480i
		OSD_DISPLAY_WIDTH  	= OSD_DISPLAY_WIDTH or HSIZE;
		OSD_DISPLAY_HEIGHT 	= OSD_DISPLAY_HEIGHT or VSIZE;
	elseif(FORMAT == "576i") then
		OSD_DISPLAY_WIDTH  	= OSD_DISPLAY_WIDTH or HSIZE;
		OSD_DISPLAY_HEIGHT 	= OSD_DISPLAY_HEIGHT or VSIZE;
	end
end 


if (FORMAT == "1080i") or (FORMAT == "1080i50") then
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
  end
  sysctl0:setVPortMode({port=0, live=VPORT_LIVE_MODE, interlaced=1, bottomfieldfirst=0, videocontrol=0x00000ba0, interleave=1});
elseif (FORMAT == "720p") or (FORMAT == "720p50") then
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  end
  sysctl0:setVPortMode({port=0, live=VPORT_LIVE_MODE, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
elseif (FORMAT == "480i") then
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  end
  sysctl0:setVPortMode({port=0, live=VPORT_LIVE_MODE, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
elseif (FORMAT == "576i") then
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  end
  sysctl0:setVPortMode({port=0, live=VPORT_LIVE_MODE, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
end
board.setVideoInput({port=0, format=FORMAT, mode=VIN});

if (PTS_120KHZ == 1) then
  sysctl0:configure({param=Q_SYS_CFG_PTS_PER_SECOND, value=120000});
end

if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
   sysctl0:setNtpInterval({sec=NTP_UPDATE_INTERVAL});
end

-- saves the input audio sample rate before the following codes changes it
AUD_FILTER_IN_FREQ = SAMPLERATE;

-- audio parameters and board setup for audio
if (VIDEO_ONLY == 0) then
   -- setup audio parameters with over-rides as necessary
   QAC_AOT = 2;
   Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
   if (AUDIO_ENCODER == "AAC-LC") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 1024;
      QAC_AOT = 2;
   elseif (AUDIO_ENCODER == "ER-AAC-LD") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 512;
      QAC_AOT = 23;
   elseif (AUDIO_ENCODER == "MPEG-LAYER-2") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QMA;
      AUDIO_FRAME_SIZE = 1152;
   elseif (AUDIO_ENCODER == "G711-A-LAW") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_A_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ENC_SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G711-MU-LAW") or (AUDIO_ENCODER == "G711") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ENC_SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G722") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q722;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 16000;
      ENC_SAMPLERATE = 16000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "PCM") then
      AUDIO_CODEC =  Q_AUDENC_CFP_CODEC_QPCM;
      if (SAMPLERATE == 32000) then
        AUDIO_FRAME_SIZE = 384;           --For low delay in segment encoding 
      else
        AUDIO_FRAME_SIZE = 512; 
      end
   elseif (AUDIO_ENCODER == "G726") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q726;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ENC_SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      if (ABITRATE ~= 40000) and (ABITRATE ~= 32000) and (ABITRATE ~= 24000) and (ABITRATE ~= 16000) then
         ABITRATE = 40000; -- set usable default
      end
   elseif (AUDIO_ENCODER == "G728") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q728;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ENC_SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 16000;
   else
      error("Invalid audio encoder " .. tostring(AUDIO_ENCODER))
   end
   if (AUD_FILTER_ENABLE == 1) then
      board.setAudioSampleFreq({port=AIN_PORT, freq=AUD_FILTER_IN_FREQ, mode=AIN});
   else
      board.setAudioSampleFreq({port=AIN_PORT, freq=SAMPLERATE, mode=AIN});
   end
end

-- audio filter's output frequency is what the audio encoder uses now (after the changes from above)
AUD_FILTER_OUT_FREQ = ENC_SAMPLERATE;

-- if the filter is an all-pass filter, the input's sample rate should be the same as the output's
if (AUD_FILTER_TYPE == Q_AUDFLTR_CFP_TYPE_ALLPASS) then
   AUD_FILTER_IN_FREQ = SAMPLERATE;
   AUD_FILTER_OUT_FREQ = SAMPLERATE;
end

AUD_FILTER_OUT_FRAMESIZE = AUDIO_FRAME_SIZE;
AUD_FILTER_IN_FRAMESIZE = AUD_FILTER_OUT_FRAMESIZE * AUD_FILTER_IN_FREQ / AUD_FILTER_OUT_FREQ;

-- there's a possibility that the output frame size of the audio filter to be too small for certain
-- kind of codec. For example, g711 doesn't like frame size to be less than about 100. 
if (AUD_FILTER_ENABLE == 1) and (AUD_FILTER_TYPE == Q_AUDFLTR_CFP_TYPE_RATECONVERSION) and
   (AUD_FILTER_OUT_FRAMESIZE < 128) then
   AUD_FILTER_OUT_FRAMESIZE = 128;
   AUD_FILTER_IN_FRAMESIZE = AUD_FILTER_OUT_FRAMESIZE * AUD_FILTER_IN_FREQ / AUD_FILTER_OUT_FREQ;
end

if (AUDIO_STEREO == 0) then
   AUD_FILTER_NUM_CHAN = 1; -- mono
else
   AUD_FILTER_NUM_CHAN = 2; -- stereo
end

--printf("IN freq %d size %d. Out freq %d size %d. Sample rate %d. Enc sample rate %d\n", AUD_FILTER_IN_FREQ, AUD_FILTER_IN_FRAMESIZE,
--AUD_FILTER_OUT_FREQ, AUD_FILTER_OUT_FRAMESIZE, SAMPLERATE, ENC_SAMPLERATE);

if (ENABLE_ANALYTICS_STREAMING == 0) then
  ws = sysctl0:createWriteStreamer({name="ws0"});
else
  -- Increase queue size as it will also contain metadata for motion events
  ws = sysctl0:createWriteStreamer({name="ws0", queuesize = 200});
end

ws:showrate({enable=SHOWRATE});
ws:start()

--metadata enable
if (ENABLE_ANALYTICS_STREAMING > 0) then
  print("Enable metadata....");
  ws:configure({param=Q_WS_CFG_METADATA, value=1});
end

--This option will turn off DMA of data from the codec to the host.  Events will still be
--exchanged for each bitstream block.
--ws:configure({param=Q_WS_CFG_READ_BITSTREAMS, value=0});

LVPP_MOTION_DETECTION = 0;
if (MOTIONDETECTION == 1) then
  LVPP_MOTION_DETECTION = 1;
end;

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16, 
                            numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, 
                            maxPMEDSFactor=MAX_PME_DOWNSCALE, maxDeintMode=DEINT_MODE,
			    burninEnable=BURNIN_AT_LVPP, privacyRegionEnable=PRIV_REGION_AT_LVPP,
                            audioInputClock=AIN_PORT, cameraTamperEnable=CAMERATAMPER,
                            motionDetectionEnable=LVPP_MOTION_DETECTION});

if (FORMAT == "1080i") or (FORMAT == "1080i50") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=563});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=20});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "720p") or (FORMAT == "720p50") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=25});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "480i") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 2, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9}); 
  if (DEINT_MODE ~= 0) then
    -- Since we are encoding only 720x480 and the encoding mode is
    -- frame, we can set the PME VERT SCALE to 2. If doing interlaced encoding
    -- we need to set it to 4.
    lvpp0:setParam({param=Q_LVPP_CMP_PMEDS_V, value = 2});
    lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=1});
  end
  lvpp0:setParam({param=Q_LVPP_CMP_DEINT_MODE, value=DEINT_MODE});
elseif (FORMAT == "576i") then
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 2, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=313});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=24});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=16});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=15});
  if (DEINT_MODE ~= 0) then
    -- Since we are encoding only 720x480 and the encoding mode is
    -- frame, we can set the PME VERT SCALE to 2. If doing interlaced encoding
    -- we need to set it to 4.
    lvpp0:setParam({param=Q_LVPP_CMP_PMEDS_V, value = 2});
    lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=1});
  end
  lvpp0:setParam({param=Q_LVPP_CMP_DEINT_MODE, value=DEINT_MODE});
end

lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=VIN_STC});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});

-- disable temporal filter for recon preview
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=TF_LUMA_BYPASS});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=TF_CHROMA_BYPASS});

if ((VIDEO_ONLY == 0) and (SEGMENT_ENCODE_MODE < 2)) then
  lvpp0:setParam({param=Q_LVPP_CMP_AV_SYNCH_ENABLE, value=1});
end

if (NUM_OF_INTERVAL_LINES ~= 0) then
  lvpp0:setParam({param=Q_LVPP_CMP_INTERVAL_INT_LINES, value = NUM_OF_INTERVAL_LINES});
end

if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
   lvpp0:setParam({param=Q_LVPP_CMP_VARIABLE_FRAME_RATE, value=1});
   lvpp0:setParam({param=Q_LVPP_CMP_NTP_CLOCK_ENABLE, value=1});
end

lvpp0:initCameraTamper({cameraTamperEnable= CAMERATAMPER});

lvpp0:activateCfg();


-- create and configure audio input
if (VIDEO_ONLY == 0) then   
   -- setup audio input
   sain0 = sysctl0:sainCreate({name="sain0",port=AIN_PORT});
   if (AUDIO_STEREO == 1) then
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_STEREO});
   else
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_MONO_L});
   end
   sain0:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
   sain0:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
   if (AUD_FILTER_ENABLE == 1) then
      sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=AUD_FILTER_IN_FREQ});
      sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUD_FILTER_IN_FRAMESIZE});
   else
      sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
      sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
   end
   sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
   if (AIN == "hdmi") then
     sain0:setParam({param=Q_SAIN_CMP_AUD_CLOCK_SOURCE, value=Q_SAIN_CFP_AUD_EXTERNAL_CLK});
  end

  if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
     sain0:setParam({param=Q_SAIN_CMP_AUD_NTP_CLOCK_ENABLE, value=1});
  end

  sain0:activateCfg();

   -- create and configure audio filter
   if (AUD_FILTER_ENABLE == 1) then
      audfil0 = sysctl0:audfilterCreate({name="audfil0", numOutputPorts=AUD_FILTER_OUTPUTS, 
					   maxFramesSize=8192, filterType=AUD_FILTER_TYPE});
      audfil0:setParam({param=Q_AUDFLTR_CMP_MAX_INPUT_PORTS, value=AUD_FILTER_INPUTS});
      -- the following parameters are for the rate conversion filtering
      audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FREQ, value=AUD_FILTER_OUT_FREQ});
      audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FRAME_NUM_SAMPLES, value=AUD_FILTER_OUT_FRAMESIZE});
      audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FREQ, value=AUD_FILTER_IN_FREQ});
      audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=AUD_FILTER_NUM_CHAN});
      audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=AUD_FILTER_IN_FRAMESIZE});
      audfil0:activateCfg();
      
      iport = audfil0:addInputPort();
      audfil0:setInputPortSource({inport=iport, src=sain0, threshold=1});
   end

   -- create and bind audio encoder
   audenc0 = sysctl0:audencCreate({name="audenc0", ws=ws, codecType=AUDIO_CODEC});
   audenc0:setStreamID({streamid=AUDIOSTREAMID});

   if (AUD_FILTER_ENABLE == 1) then
      audenc0:bindAudioInput({src=audfil0, port=0}); -- binds to output port 0 of audio filter
   else
      audenc0:bindAudioInput({src=sain0});
   end

   -- configure audio encoder
   if (AUDIO_STEREO == 1) then
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_STEREO});
   else
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_MONO});
   end
   if (AUD_FILTER_ENABLE == 1) then
      audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=AUD_FILTER_OUT_FREQ});
   else
      audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=SAMPLERATE});
   end
   audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
   audenc0:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
   --codec specific configuration
   audenc0:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q711_LAW});
   audenc0:setParam({param=Q_AUDENC_CMP_QAC_AUDIO_OBJECT_TYPE, value=QAC_AOT});

   if(STREAM_FORMAT == 2) then
       audenc0:setParam({param=Q_AUDENC_CMP_TS_ENABLE, value=1});
       audenc0:setParam({param=Q_AUDENC_CMP_TS_STREAM_ID, value=1});
       audenc0:setParam({param=Q_AUDENC_CMP_TS_PCR_MASTER, value=1});
   end
   audenc0:activateCfg();
end

if (FORMAT == "720p")  or (FORMAT == "720p50") then
	raw_rate = 330000000 * 3003/VIN_OUTPUT_STC  ;
elseif (FORMAT == "1080i")  or (FORMAT == "1080i50") then
	raw_rate = 746000000 * 3003/VIN_OUTPUT_STC  ;
else
	raw_rate = 125000000 * 3003/VIN_OUTPUT_STC  ;
end

percentage_rate = 100 * VBITRATE/raw_rate;
print("Raw bitrate", raw_rate, "bitrate", VBITRATE, "percentage rate", percentage_rate);

avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, numreferenceframes=NUM_AVC_REFERENCES, outputbuffersize=ENC_OUTPUT_BUFFER});

-- setup avc
--avcenc_VSIZEi60(avcenc, profile, level, bitrate_qp, gop_structure, picture_coding)
if (FORMAT == "1080i") then
	if(percentage_rate<0.6 and QUALITY_OVERRIDE == 0) then
  		avcenc_1080i60_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  	else
  		avcenc_1080i60(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  	end
elseif (FORMAT == "1080i50") then
	if(percentage_rate<0.6 and QUALITY_OVERRIDE == 0) then
  		avcenc_1080i60_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	else
		avcenc_1080i60(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  end
elseif (FORMAT == "720p") then
 	if(percentage_rate<0.55 and QUALITY_OVERRIDE == 0) then
  		avcenc_720p60_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	else
  		avcenc_720p60(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	end
elseif (FORMAT == "720p50") then
	if(percentage_rate<0.55 and QUALITY_OVERRIDE == 0) then
		avcenc_720p60_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	else
		avcenc_720p60(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	end
elseif (FORMAT == "480i") then
  if (COMPATIBLE1264 == 1) then
    if(gopstruct ~= Q_AVCENC_CFP_GOP_STRUCTURE_I ) and (gopstruct ~= Q_AVCENC_CFP_GOP_STRUCTURE_IP) then
      print("Error, invalid GOP structure used for 1264 compatible streams");
    else
      avcenc_480_1264compatible(avcenc0, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
    end
  elseif (HIGHQUALITY == 1) then
  	if(percentage_rate<1.5 and QUALITY_OVERRIDE == 0) then
		avcenc_480i_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	else
    	avcenc_480i_highquality(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	end
  else
  	if(percentage_rate<1.5 and QUALITY_OVERRIDE == 0) then
    	avcenc_480i_lowbr(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  	else
		avcenc_480i(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
	end
  end
elseif (FORMAT == "576i") then
  -- The 480i settings work for 576i except for the clock ticks
  if (HIGHQUALITY == 1) then
    	avcenc_480i_highquality(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  else
    avcenc_480i(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, gopstruct, FRAME_STRUCTURE)
  end
end

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_INVERSE_TELECINE_ENABLE, value=INVERSE_TELECINE});


--Changing frame rate for low bitrate video.
if(QUALITY_OVERRIDE == 0) then
	if(FORMAT=='480i') then
		if(percentage_rate < 0.8) then
			print("Bitrate too low. Enforcing quality constraints, use QUALITY_OVERRIDE = 1 to overide.");
			stc = VIN_OUTPUT_STC;
			VIN_OUTPUT_STC =  math.ceil( (1.48-0.6*percentage_rate)*stc );
			if(percentage_rate < 0.60 ) then
				VIN_OUTPUT_STC =  math.ceil( (1.056/percentage_rate-0.64)*stc );
			end
			frame_rate = 90000/VIN_OUTPUT_STC;
			print("...Extreme low bitrate, reducing frame rate to ",frame_rate);
			lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
			avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=VIN_OUTPUT_STC});
		end
	elseif(FORMAT=='720p' or FORMAT=='720p50') then	
		--Frame rate reduction for HD
		if(percentage_rate < 0.55) then
			print("Bitrate too low. Enforcing quality constraints, use QUALITY_OVERRIDE = 1 to overide.");
			stc = VIN_OUTPUT_STC;
			VIN_OUTPUT_STC = 3000;
			frame_rate = 90000/VIN_OUTPUT_STC;
			print("...Extreme low bitrate, reducing frame rate to ",frame_rate);
			lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
			avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=VIN_OUTPUT_STC});
	 		end
	else
		if(percentage_rate<0.6) then
			print("Bitrate too low. Enforcing quality constraints, use QUALITY_OVERRIDE = 1 to overide.");
			stc = VIN_OUTPUT_STC;
				VIN_OUTPUT_STC =  math.ceil( 90000/(50*percentage_rate) );
			if(percentage_rate<0.4) then
				VIN_OUTPUT_STC = math.ceil( 90000/(35.7*percentage_rate+5.7));
			end
			if(percentage_rate<0.26) then
				VIN_OUTPUT_STC = 6000;
			end
			frame_rate = 90000/VIN_OUTPUT_STC;
			print("...Extreme low bitrate, reducing frame rate to ",frame_rate);
			lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
			avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=VIN_OUTPUT_STC});
		end
	end
	lvpp0:activateCfg();
	avcenc0:activateVideoEncCfg();
end


avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, value=ENC_GUARD_BAND_PER_MB});

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=VIN_OUTPUT_STC});

if (NUM_OF_INTERVAL_LINES ~= 0) then
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_SEGMENT_ENCODE_ENABLE, value=1});
  if(FORMAT == "1080i") then
    for i=0, 7,1 do
      avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_MEREGION_MVSEL16X16,value=3});
      avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_MEREGION_MVSEL16X8,value=3});
      avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_MEREGION_MVSEL8X16,value=3});
      avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_MEREGION_MVSEL8X8,value=3});
    end
  end
end

if (STREAMING > 0) then
  -- Set the rate control to reduce I frame size, useful for low latency streaming
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_I_DELTA_VARIABLE, value=0});
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_I_DELTA_MIN, value=0});
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_I_DELTA_MAX, value=0});
  -- Set the rate control buffer model to reduce bitrate variation, because the default settings
  -- are very tolerant.
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_BUFFER_TRANSFER_RATE_BITS, value=(VBITRATE * 2)});
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_BUFFER_INITIAL_DELAY, value=(VIN_STC * 10)});
  -- Complexity and static detection are actually useful for most applications, but they are
  -- especially important to prevent low motion scenes from being followed by huge I frames.
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_COMPLEXITY_ENABLE, value=1});
  -- Limit the maximum quality to QP 20.  Streaming normally uses a low bitrate, so the only change
  -- this should make is to prevent quality from becoming too high on
  avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_QP_RANGE_MIN, value=20});
end

--[[
avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_ADAPTIVE_FRAMERATE_ENABLE, value=1});
avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_AFR_MIN_QP, value=32});
avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_AFR_MAX_QP, value=35});
avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_AFR_SCALING_DENOMINATOR, value=30});
avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_AFR_SCALING_MIN_NUMERATOR, value=1});
]]--
--avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_BASE_QP, value=51});
--]]


if (MOTIONDETECTION == 2) then

  -- First, enable motion detection flag inside encoding parameters
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MOTION_DETECT_ENABLE, value=1});

  -- Activates preproc delay to enable motion detection to work properly with other preprocessing features
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_DELAY_PREPROC, value=PREPROC_DELAY_ENABLE});

  --In case of analytic streaming, GUI will create the regions by calling startMotionDetection
  if (ENABLE_ANALYTICS_STREAMING == 0) then
    -- add 9 motion regions in a 3x3 grid
    i = 0;
    HMAX=math.floor(HSIZE/16);
    VMAX=math.floor(VSIZE/16);
    HINCR=math.floor(HMAX/3);
    VINCR=math.floor(VMAX/3);
    print("HMAX "..HMAX.." VMAX "..VMAX.." HINCR "..HINCR.." VINCR "..VINCR);
    xpos = 0;
    ypos = 0;
    for y = 0, 2, 1 do 
      for x = 0, 2, 1 do
        -- if we are at the right or left we add in right/bottommost blocks so the 
        -- whole screen is covered
        if (x == 2) then
          hdelta = HMAX % 3;
        else
          hdelta = 0;
        end
        if (y == 2) then
          vdelta = VMAX % 3;
        else
          vdelta = 0;
        end
        print("roi x "..xpos.." y "..ypos.." hdelta "..hdelta.." vdelta "..vdelta);
        avcenc0:addMDROI({index=i, width=HINCR+hdelta, height=VINCR+vdelta, offsetX=xpos, offsetY=ypos});
        avcenc0:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_ENABLE, value=1});
        avcenc0:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_MOTION_THRESHOLD, value=5});
        avcenc0:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_PERCENT, value=200});
        i = i + 1;
        xpos = xpos + HINCR;
      end
      xpos = 0;
      ypos = ypos + VINCR;
    end
  end

  if (ADAPTIVE_BITRATE_MOTIONDETECTION_ENABLE == 1) then
    -- enable adaptive motion
    avcenc0:setAdaptiveBitrateByMotion({enable=1,
                                        lobitrate=ADAPTIVE_BITRATE_MOTIONDETECTION_LO_RATE,
                                        hibitrate=ADAPTIVE_BITRATE_MOTIONDETECTION_HI_RATE,
                                        loframeticks=ADAPTIVE_BITRATE_MOTIONDETECTION_LO_FRAMETICKS,
                                        hiframeticks=ADAPTIVE_BITRATE_MOTIONDETECTION_HI_FRAMETICKS,
                                        duration=ADAPTIVE_BITRATE_MOTIONDETECTION_DURATION});

  end
end


if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NTP_ENABLE, value=1});
end

avcenc0:activateVideoEncCfg();

avcenc0:bindVideoInput({src=lvpp0});

avcenc0:setStreamID({streamid=VIDEOSTREAMID});

-- ack every N frames to the AVC encoder
avcenc0:setAckInterval({interval=3});

if (MOTIONDETECTION == 1) then
  -- Enables motion detection
  lvpp0:setPreProcParam({param=Q_LVPP_CMP_PREPROC_MOTION_DETECT_ENABLE, value=1});

  -- Activates preproc delay to enable motion detection to work properly with other preprocessing features
  lvpp0:setPreProcParam({param=Q_LVPP_CMP_PREPROC_DELAY_PREPROC, value=PREPROC_DELAY_ENABLE});
  
  i = 0;
  HMAX=math.floor(HSIZE/16);
  VMAX=math.floor(VSIZE/16);
  HINCR=math.floor(HMAX/3);
  VINCR=math.floor(VMAX/3);
  print("HMAX "..HMAX.." VMAX "..VMAX.." HINCR "..HINCR.." VINCR "..VINCR);
  xpos = 0;
  ypos = 0;
  for y = 0, 2, 1 do 
     for x = 0, 2, 1 do
       -- if we are at the right or left we add in right/bottommost blocks so the 
       -- whole screen is covered
       if (x == 2) then
         hdelta = HMAX % 3;
       else
         hdelta = 0;
       end
       if (y == 2) then
         vdelta = VMAX % 3;
       else
         vdelta = 0;
       end
       print("roi x "..xpos.." y "..ypos.." hdelta "..hdelta.." vdelta "..vdelta);
       lvpp0:addMDROI({index=i, width=HINCR+hdelta, height=VINCR+vdelta, offsetX=xpos, offsetY=ypos});
       lvpp0:setMotionDetectParam({idx=i, param=Q_LVPP_CMP_MOTION_DETECT_ROI_ENABLE, value=1});
       lvpp0:setMotionDetectParam({idx=i, param=Q_LVPP_CMP_MOTION_DETECT_ROI_MOTION_THRESHOLD, value=5});
       lvpp0:setMotionDetectParam({idx=i, param=Q_LVPP_CMP_MOTION_DETECT_ROI_PERCENT, value=200});
       i = i + 1;
       xpos = xpos + HINCR;
    end
    xpos = 0;
    ypos = ypos + VINCR;
  end 

  lvpp0:activateCfg();
end

-- Load font resource
if (BURNIN_TIMECODE ~= 0) then
  -- check if font file exists and if not then turn off burnin
  if (io.open(BURNIN_FONT[BURNIN_FONTSIZE]) == nil) then
    print("Warning: Could not load font file "..BURNIN_FONT[BURNIN_FONTSIZE].."\n");
    BURNIN_TIMECODE = 0;
  else
    BurnInFontAddr = sysctl0:allocBurnInFont({fontsize=BURNIN_FONTSIZE});
    sysctl0:loadBurnInFont({fontaddr=BurnInFontAddr, fontsize=BURNIN_FONTSIZE, filename=BURNIN_FONT[BURNIN_FONTSIZE]});
  end
end

-- Enable burnin if asked
if (BURNIN_TIMECODE ~= 0) then
  if (BURNIN_AT_LVPP == 0) then
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_ENABLE, value=1});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FONT_SIZE, value=BURNIN_FONTSIZE});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_UPD_MODE, value=0});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_X, value=128});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_Y, value=96});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FRAME_NUM_ENABLE, value=1});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FRAME_NUM_POS, value=16});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FRAME_NUM_L, value=0});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_FRAME_NUM_H, value=30});
    avcenc0:setBurnInParam({index=0, param=Q_AVCENC_CMP_BURNIN_STR_LEN, value=18});
    avcenc0:setBurnInText({index=0, text="Frame Counter : FF"});
    avcenc0:activateVideoEncCfg();

    avcenc0:setBurnInParam({index=1, param=Q_AVCENC_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
    avcenc0:setBurnInParam({index=1, param=Q_AVCENC_CMP_BURNIN_FONT_SIZE, value=BURNIN_FONTSIZE});
    avcenc0:setBurnInAutoTime({index=1, enable=1, fps=30, x=128, y=128, droptime=0});
  else
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_ENABLE, value=1});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FONT_SIZE, value=BURNIN_FONTSIZE});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_UPD_MODE, value=0});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_X, value=128});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_Y, value=96});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FRAME_NUM_ENABLE, value=1});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FRAME_NUM_POS, value=16});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FRAME_NUM_L, value=0});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_FRAME_NUM_H, value=30});
    lvpp0:setBurnInParam({index=0, param=Q_LVPP_CMP_BURNIN_STR_LEN, value=18});
    lvpp0:setBurnInText({index=0, text="Frame Counter : FF"});
    lvpp0:activateCfg();

    lvpp0:setBurnInParam({index=1, param=Q_LVPP_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
    lvpp0:setBurnInParam({index=1, param=Q_LVPP_CMP_BURNIN_FONT_SIZE, value=BURNIN_FONTSIZE});
    lvpp0:setBurnInAutoTime({index=1, enable=1, fps=30, x=128, y=128, droptime=0});
  end
     
end

-- General audio/video preview setup
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  if(OSD_ENABLE == 1) then
  	avoc0 = sysctl0:avocCreate({name="avoc0", gfx0Size=OSD_MEMORY_SIZE, allocGfx=1});
  else 
  	avoc0 = sysctl0:avocCreate({name="avoc0"});
  end
  avsn0 = sysctl0:avsnCreate({name="avsn0"});
  if (FORMAT == "1080i") then
    avoc0:setOutput1080i60({sync=vout_sync});
  elseif (FORMAT == "1080i50") then
    avoc0:setOutput1080i50({sync=vout_sync});
  elseif (FORMAT == "720p") then
    avoc0:setOutput720p60();
  elseif (FORMAT == "720p50") then
    avoc0:setOutput720p50();
  elseif (FORMAT == "480i") then
    avoc0:setOutput480i();
  elseif (FORMAT == "576i") then
    avoc0:setOutput576i();
  end

  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});

  avoc0:setParam({param=Q_AVOC_CMP_VID_OUTPUT_TICKS, value=VIN_STC});

  avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
  avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
  if (AUD_FILTER_ENABLE == 1) then
     avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=AUD_FILTER_IN_FREQ});
  else
    avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
  end
  avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});

  if (ADAPTIVE_BITRATE_MOTIONDETECTION_ENABLE == 1) then
    -- if we are using adaptive motion then we want to disable deinterlacing by the avoc
    -- as will be reducing the framerate
    avoc0:setParam({param=Q_AVOC_CMP_EXTERNAL_DEINTERLACER, value=1});
    avoc0:activateCfg();
  end

  avoc0:activateCfg();

  vch0 = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
  avoc0:activateVideoChannelCfg();

  if(OSD_ENABLE == 1) then
    osdinit({ avoc=avoc0, 
                width=OSD_DISPLAY_WIDTH, 
                height=OSD_DISPLAY_HEIGHT,
                x=OSD_X_OFFSET,y=OSD_Y_OFFSET,
                memory=OSD_MEMORY_SIZE,
              });
  end

  if (VIDEO_ONLY == 0) then
    -- Echo audio to output
    avsn0:bindAudioInput({src=sain0});
    if (AUDIO_STEREO == 1) then
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
    else
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
    end
    avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
    avoc0:enableAudioChannel({ch=ach0, enable=1});
    avoc0:activateAudioChannelCfg();
  end

  -- Activate live or reconstructed video preview
  if (PREVIEW == 1) then
    avsn0:bindVideoInput({src=lvpp0});

    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=0});

    if (VIDEO_ONLY == 0) then
      avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=0});
    else
      avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=VIN_OUTPUT_STC * -3});
    end
  elseif (PREVIEW == 2) then
    avsn0:bindVideoInput({src=avcenc0});
 
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=1});

    avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=VIN_OUTPUT_STC * -1 * AVSN_DELAY});
  end
  avsn0:activateCfg();

  avcenc0:activateVideoEncCfg();
end

if (STREAMING > 0 or CIRCULAR_SIZE > 0) then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=1});
    if (STREAMING > 0) then
      avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
      -- VLC does not support field coding so we have to turn off bottom field
      -- coding on intra frames
      avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_IP_FIELD_CODING, value=0});
    end
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
    if(CIRCULAR_SIZE > 0) then
      --Following is needed if you want to convert to MP4 using the streamexport tools
      avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_QMED_ENABLE, value=1});
    end
    avcenc0:activateVideoEncCfg();
end

if (STREAM_FORMAT == 2) then

  sysctl0:configure({param=Q_SYS_CFG_PACKETS_PACK_NUMBER, value=TS_PACK_NUMBER});
 -- real pts will be set value + 1 frame pts
  if (VIDEO_ONLY == 0) then
    sain0:setParam({param=Q_SAIN_CMP_AUD_START_PTS_LOW, value=0x337f3a4});
    sain0:activateCfg();
  end
  lvpp0:setParam({param=Q_LVPP_CMP_START_PTS_LOW, value=0x337f3a4});
  lvpp0:activateCfg();

  if (FORMAT == "720p") then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_START_DTS_LOW, value=0x337f3a4});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_DTS_INTERVAL, value=VIN_OUTPUT_STC});
  elseif (FORMAT == "1080i") or (FORMAT == "480i") or (FORMAT == "576i") then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_START_DTS_LOW, value=0x337edc5});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_DTS_INTERVAL, value=VIN_OUTPUT_STC});
  end
 
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_TS_ENABLE, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_AU_DELIMITERS, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_TS_STREAM_ID, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_TS_PCR_MASTER, value=1});
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_OUTPUT_AU_ALIGNMENT, value=0});

  os.execute("echo 1 0x014 4 0x57e7d > /proc/driver/qcc/writebid");

end

-- start output
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0:start();
end

record();

if (FILE_STREAMER == 1) then
  fs = sysctl0:createfilestreamer({name="fs", serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR})
  fs:open({filename="qboxfile"});

  fs:open({filename="h264_480i_aac.qbx"});

  fs:open({filename="Qbx_mjpeg.qbx"});
end


-- This function is deprecated and replaced by the stop() function.
-- Kept here for backward compatibility in case some test is using this
-- but changed to use the stop() function
function done()
  stop();
end

-- Pause encoding
function pause()
    lvpp0:stop();
end

-- Resume playback from paused state
function resume()
    lvpp0:resume();
end  

-- Specify the number of ticks per frame to lower framerate (3003 = 29.97 fps, 6006 for 15fps, etc)
function framerate(rate)
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=rate});
    avcenc0:activateVideoEncCfg();
end

function query() 
  avcenc0:query()
  lvpp0:query()
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    avoc0:query()
    avsn0:query()
  end
  sysctl0:query()
  ws:query()
end

-- Function to change resolution on the fly. 
function resolutionChange(outwidth,outheight)
  	print("change resolution to width = ,height =  ...\n",outwidth,outheight);
 	avoc0:setParam({param=Q_AVOC_CMP_VID_0_SCALING_ENABLE, value=0}); 
 	avoc0:activateCfg();
 	lvpp0:setParam({param=Q_LVPP_CMP_PICT_WIDTH, value=outwidth});
  	lvpp0:setParam({param=Q_LVPP_CMP_PICT_HEIGHT, value=outheight});
  	lvpp0:activateCfg();
end

-- Function to change bitrate on the fly.
function bitratechange(bitrate)
	avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_BITRATE, value=bitrate});
    avcenc0:activateVideoEncCfg();
end

-- Function to change GOP size on the fly.
function gopsizechange(gopsize)
	avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_GOP_SIZE, value=gopsize});
    avcenc0:activateVideoEncCfg();
end

-- Function to change sigature on the fly.
function sigchange(signature)
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_START_SIGNATURE, value=signature});
    avcenc0:activateVideoEncCfg();
end

if (CIRCULAR_SIZE > 0) then
  -- Function to save pre and post recordings from circular writer
  -- Call with base filename without .qbx
  -- For example, if called with /data/test, the files recorded will be
  -- /data/testpre.qbx and /data/testpost.qbx
  function saveprepostbuf(basefilename)
    if(STREAM_FORMAT ~= 2) then
      prefilename=basefilename.."pre.qbx"
      postfilename=basefilename.."post.qbx"
    else
      prefilename=basefilename.."pre.ts"
      postfilename=basefilename.."post.ts"
    end
    ws:ioctl({bsw=bswc,cmd=Q_WS_IOCTL_CIRCULARWRITER_SAVE,
      arg=string.format("prefilename=%s,postfilename=%s", prefilename, postfilename)})
    if(STREAM_FORMAT ~= 2) then
      -- Use streamexport here to convert the resulting QBOX file
      -- to MP4 or MKV format
      out_filename=string.format("%s.%s",basefilename, STREAMEXPORT_FORMAT);
      if (POST_SIZE == 0) then
        cmd_string=string.format("streamexport --type %s -o %s %s",
                STREAMEXPORT_FORMAT, out_filename, prefilename);
      else
        cmd_string=string.format("streamexport --type %s -o %s %s %s",
                STREAMEXPORT_FORMAT, out_filename, prefilename, postfilename);
      end
      print(cmd_string);
      os.execute(cmd_string);
    else
      -- Don't do any thing if only pre-recording
      -- Concatenate in case of both pre and post recording
      if (POST_SIZE ~= 0) then
        out_filename=string.format("%s.ts",basefilename);
        cmd_string=string.format("rm -f %s", out_filename);
        print(cmd_string);
        os.execute(cmd_string);
        cmd_string=string.format("cat %s %s > %s",
                prefilename, postfilename, out_filename);
        print(cmd_string);
        os.execute(cmd_string);
      end
    end
  end
end

if (ENABLE_ANALYTICS_STREAMING > 0) then
  function stopMotionDetection(avcenc0, old_index, old_xoff, old_yoff, old_height, old_width)
    print("index, width, height ...\n",old_index,old_width,old_height);

    --  Removing region of interest
    avcenc0:subMDROI({index=old_index, width=old_width, height=old_height, offsetX=old_xoff, offsetY=old_yoff});
  end

  function startMotionDetection(avcenc0,new_index, xoff, yoff, new_height, new_width)
    os.execute ("echo Setting motion detection region");
    
    print("index, xoff, yoff, width, height ...\n",new_index,xoff,yoff,new_width,new_height);
    -- Here we have to ask user for newer x,y,height, width information
    avcenc0:addMDROI({index=new_index, width=new_width, height=new_height, offsetX=xoff, offsetY=yoff});
    avcenc0:setVideoEncROIParam({idx=new_index, param=Q_AVCENC_CMP_ROI_ENABLE, value=1});
    avcenc0:setVideoEncROIParam({idx=new_index, param=Q_AVCENC_CMP_ROI_MOTION_THRESHOLD, value=5});
    avcenc0:setVideoEncROIParam({idx=new_index, param=Q_AVCENC_CMP_ROI_PERCENT, value=2000});

    avcenc0:activateVideoEncCfg();
  end
end


