
--[[
	This script is to stream linear qbox file using rtsp/rtp stack.
	Following are discription of lua calls, it's arguments and it's usage.

	+ To create filestreamer object. All subsequent calls use this object. 
		luacall- sysctl0:createfilestreamer({name, serverinfo, mountpoint}) 
		mandatory args- name - unique name for filestreamer object
						serverinfo - rtsp server information (like server ip, server port, mod, max segament size)
						mountpoint - mount path to search qboxfiles
		optional args- none
		Example: fs = sysctl0:createfilestreamer({name="fs", serverinfo="sip=NULL,prt=554,mod=0,msz=1472", mountpoint=/mount/point/path/})

	+ To open filestreamer object. This call will create static RTSP session and start filestreaming on rtsp request.
	  It's optional call. If Dynamic session addion feature is supported, than all files under specified mountpoint (in createfilestreamer call) 
	  can be stream without using this call.
		luacall- fsobj:open({filename, looping, zerocopy, srcId})
		mandatory args- filename - unique name of qbox file. RTSP session name is same as filename.
		optional args- 	looping - flag to enable/disable file looping feature. (default is 0 means disable)
						zerocopy - flag to enable/disable zerocopy feature. (default is 1 means enable)
									zerocopy improves sytem performance.
									constraint: Either zerocopy or looping can be enabled. If zerocopy is enabled, looping won't be in effect.
						srcId - stream ids. filestreamer will stream only these streams. (default is none means all streams)
		Example: fs:open({filename="qboxfilename.qbx"}) OR
				 fs:open({filename="qboxfilename.qbx", zerocopy=0}) OR
				 fs:open({filename="qboxfilename.qbx", looping=1, zerocopy=0}) OR
				 fs:open({filename="qboxfilename.qbx", srcId={videostreamid, audiostreamid,...}}) OR
				 fs:open({filename="qboxfilename.qbx", srcId={videostreamid, audiostreamid,...}, zerocopy=0}) OR
				 fs:open({filename="qboxfilename.qbx", looping=1, zerocopy=0, srcId={videostreamid, audiostreamid,...}})

	+ To close filestreamer object. This call will close rtsp session.
		luacall- fsobj:close({filename})
		mandatory args- none
		optional args- filename - filename or sessionname for which filestreaming should close. (default is null means close all sessions in this object)
		Example: fs:close({}) OR
				 fs:close({filename="qboxfilename.qbx"})

	+ To destroy filestreamer object. This call will destroy filestreamer object. After this call filestreamer object can not be used for any call.
		luacall- fsobj:destroy()
		mandatory args- none
		optional args- none
		Example: fs:destroy()
		
]]--


-- For linear qbox file
VIDEOSTREAMID = 2;
AUDIOSTREAMID = 1;

FILE_STREAMER_MOUNT_DIR_BASE = "/directmnt/qa/streams/merlin/"

fs = sysctl0:createfilestreamer({name="fs",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ibbrbp-mbaff-high-32k-30fps/" });
fs:open({filename="h264_720p_aac-lc.qbx", looping=0, zerocopy=1})
--fs:open({filename="h264_720p_g711-a-law.qbx"})
fs:open({filename="h264_720p_g711-mu-law.qbx", looping=0, zerocopy=1})
--fs:open({filename="h264_720p_g726.qbx"})
fs:open({filename="h264_720p.qbx", looping=0})
fs:open({filename="h264_1080i_aac-lc.qbx", looping=0, zerocopy=1})
--fs:open({filename="h264_1080i_g711-a-law.qbx"})
fs:open({filename="h264_1080i_g711-mu-law.qbx", looping=0, zerocopy=1})
--fs:open({filename="h264_1080i_g726.qbx"})
fs:open({filename="h264_1080i.qbx", looping=0, zerocopy=1})

fs1 = sysctl0:createfilestreamer({name="fs1",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ibbrbp-mbaff-main-48k-60fps/" });
fs1:open({filename="h264_480i_aac-lc.qbx", looping=1, zerocopy=0, srcId={AUDIOSTREAMID}});
--fs1:open({filename="h264_480i_g711-a-law.qbx"})
--fs1:open({filename="h264_480i_g711-mu-law.qbx", looping=1, zerocopy=0})
--fs1:open({filename="h264_480i_g726.qbx"})
fs1:open({filename="h264_480i.qbx", looping=0, zerocopy=1})

fs2 = sysctl0:createfilestreamer({name="fs2",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-baseline-16k-30fps/" });
--fs2:open({filename="h264_480i_aac-lc.qbx", looping=1, zerocopy=0})
fs2:open({filename="h264_480i_g711-a-law.qbx"})
--fs2:open({filename="h264_480i_g711-mu-law.qbx", looping=1, zerocopy=0})
fs2:open({filename="h264_480i_g726.qbx"})
--fs2:open({filename="h264_480i.qbx", looping=1, zerocopy=0})

fs3 = sysctl0:createfilestreamer({name="fs3",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-high-32k-25fps/" });
fs3:open({filename="h264_576i_aac-lc_pal.qbx", looping=0, zerocopy=1})

fs4 = sysctl0:createfilestreamer({name="fs4",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-high-32k-30fps/" });
--fs4:open({filename="h264_480i_aac-lc.qbx", looping=1, zerocopy=0})
--fs4:open({filename="h264_480i_g711-a-law.qbx"})
fs4:open({filename="h264_480i_g711-mu-law.qbx", looping=0, zerocopy=1})
--fs4:open({filename="h264_480i_g726.qbx", srcId={VIDEOSTREAMID}})
--fs4:open({filename="h264_480i.qbx", looping=1, zerocopy=0})

fs5 = sysctl0:createfilestreamer({name="fs5",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-high-32k-50fps/" });
fs5:open({filename="h264_576i_aac-lc_pal.qbx", looping=0, zerocopy=1})

fs6 = sysctl0:createfilestreamer({name="fs6",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-high-32k-varfps/" });
fs6:open({filename="h264_480i_aac-lc_varfps.qbx", looping=0, zerocopy=1})

fs7 = sysctl0:createfilestreamer({name="fs7",serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE .. "filestreamer/ip-frame-high-48k-60fps/"});
--fs7:open({filename="h264_480i_aac-lc.qbx", looping=1, zerocopy=0})
--fs7:open({filename="h264_480i_g711-a-law.qbx"})
--fs7:open({filename="h264_480i_g711-mu-law.qbx", looping=1, zerocopy=0})
--fs7:open({filename="h264_480i_g726.qbx"})
--fs7:open({filename="h264_480i.qbx", looping=1, zerocopy=0})
--fs7:open({filename="h264_720p_aac-lc.qbx", looping=1, zerocopy=0})
fs7:open({filename="h264_720p_g711-a-law.qbx"})
--fs7:open({filename="h264_720p_g711-mu-law.qbx", looping=1, zerocopy=0})
fs7:open({filename="h264_720p_g726.qbx"})
--fs7:open({filename="h264_720p.qbx", looping=0})
--fs7:open({filename="h264_1080i_aac-lc.qbx", looping=1, zerocopy=0})
fs7:open({filename="h264_1080i_g711-a-law.qbx"})
--fs7:open({filename="h264_1080i_g711-mu-law.qbx", looping=1, zerocopy=0})
fs7:open({filename="h264_1080i_g726.qbx"})
--fs7:open({filename="h264_1080i.qbx", looping=1, zerocopy=0})

fs8 = sysctl0:createfilestreamer({name="fs8", serverinfo=string.format("sip=%s,prt=%d,mod=%d,msz=1472", "NULL", 554, 0), mountpoint=FILE_STREAMER_MOUNT_DIR_BASE});
fs8:open({filename="spider.480i.qbox", looping=0, zerocopy=1})


-- os.execute("sleep 10");
--printf("closing all sessions under fs object \n");
-- to close all sessions under fs object
--fs:close({});
-- os.execute("sleep 10");
--printf("destroying fs object \n");
-- to destroy fs object
--fs:destroy();
--printf("closing one stream in fs1\n");
-- to close only one session under fs1 object
--fs1:close({filename="h264_480i_aac-lc.qbx"});

