
--[[

This script is used to decode Motion-JPEG bitstreams from network or HDD.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------
-- Video output mode
-- "hdmi" HDMI output 1080i and 720p only)
-- "component" Component output
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT        = VOUT or "svideo";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
-- "576i"
FORMAT      = FORMAT or "480i";

-- Audio Codec (cannot be specified with -e option as the value passed will always be 0; change here or use another configuration Lua script)
-- Q_AVDEC_CFP_AUDIO_CODEC_QAC
-- Q_AVDEC_CFP_AUDIO_CODEC_Q711
-- Q_AVDEC_CFP_AUDIO_CODEC_Q722
-- Q_AVDEC_CFP_AUDIO_CODEC_Q726
VIDEO_CODEC = VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_JPEG;
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QAC;
AUDIO_STEREO = AUDIO_STEREO or 1;  -- Use 0 for Q_AVDEC_CFP_AUDIO_CODEC_Q711

-- Audio sample rate and stereo/mono (depending on codec)
if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QAC) then
  SAMPLERATE  = SAMPLERATE or 32000;
  AUDIO_STEREO = AUDIO_STEREO or 1;
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q711) or (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q726) then
  SAMPLERATE  = 8000;
  AUDIO_STEREO = 0;
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q722) then
  SAMPLERATE = 16000;
  AUDIO_STEREO = 0;
end

-- 0 for none, 1 for UDP. RTP not supported
STREAMING = STREAMING or 0;
-- Port number used in UDP
PORT = PORT or 9700;
-- UDP buffer size
UDP_BUFFERSIZE = UDP_BUFFERSIZE or 4000000;

SERVER_IP = SERVER_IP or "10.32.84.176";

SERVER_PORT = SERVER_PORT or 554;

-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test";

-- RTP Data protocol - "udp" or "tcp"
RTP_DATA_PROTO = RTP_DATA_PROTO or "udp";

-- Client Port number
CLIENT_PORT = CLIENT_PORT or 1024;

-- Control frame drops. 
-- N/M=2/3 means drop 1 frame every 3 frames.
-- N=M means no drop
FRAME_DROP_RATIO_N = FRAME_DROP_RATIO_N or 1;
FRAME_DROP_RATIO_M = FRAME_DROP_RATIO_M or 1;

-- Override default frame rate
-- 0: use default
FRAME_TIME_OVERRIDE = FRAME_TIME_OVERRIDE or 0;

-- Input file format
-- 0 for qbox type jpeg file
-- 1 for elementary jpeg file (ie, no qbox header)
INFILE_FORMAT = INFILE_FORMAT or 1;

-- Input (m)jpeg files (only choose one)
INFILE = INFILE or
-- Files on "/directmnt/": 

-- Elementary jpeg files (still images):
--"/directmnt/qa/streams/merlin/1280x720_Ice_Hockey.jpg";
--"/directmnt/qa/streams/merlin/1280x720_Toys_Calendar.jpg";

-- Elementary Motion-jpeg files:
--"/directmnt/qa/streams/merlin/racecar_480.mjpeg"; --this video has very high bitrate that slows down display
--"/directmnt/qa/streams/merlin/BigShips1280x720.mjpeg";
--"/directmnt/qa/streams/merlin/whale22mbps_1920x1080.mjpeg";
--"/directmnt/qa/streams/merlin/whale33mbps_1920x1080.mjpeg";
--"/directmnt/qa/streams/merlin/MobileCalendar_176x144.mjpeg";
--"/directmnt/qa/streams/merlin/Whale_Show_160x120.mjpeg";
"/directmnt/qa/streams/merlin/Whale_Show_720x480.mjpeg";

-- QBox MJPEG file:
--"/directmnt/qa/streams/merlin/spider480.mjpeg.qbox";

-- Allow video only decode
VIDEO_ONLY = VIDEO_ONLY or 1;

-- To display update message every second
SHOWUPDATE = SHOWUPDATE or 0;

-- Customized the aspect ratio of the displayed video
ASPECT_RATIO_WIDTH = ASPECT_RATIO_WIDTH or nil;
ASPECT_RATIO_HEIGHT = ASPECT_RATIO_HEIGHT or nil;

-- Direct memory transfer mode (aka JPEG instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (MEMORY_XFER_MODE == 0) then
   VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_JPEG;
else
   VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_JPEG_INSTANT_DISPLAY;
end

if (STREAMING == 1) then
   UDP_QBOX = 0;    --1 for SDK5x; 0 for SDK6x
else
   UDP_QBOX = 0;
end

if (INFILE_FORMAT == 0) then
   JPEG_READER_TYPE = "qbox";
else
   JPEG_READER_TYPE = "jpeg";
end

if (FORMAT == "720p") then
  HSIZE = 1280;
  VSIZE = 720;
  AR_X = 1;
  AR_Y = 1;
  VIDEO_FRAME_TIME = 1500;
elseif (FORMAT == "1080i") then
  HSIZE = 1920;
  VSIZE = 1080;
  VIDEO_FRAME_TIME = 3003;
  AR_X = 1;
  AR_Y = 1;
elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
  AR_X = 8;
  AR_Y = 9;
  VIDEO_FRAME_TIME = 3003;
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  VIDEO_FRAME_TIME = 3003;
  AR_X = 8;
  AR_Y = 9;
else
  print("Invalid format\n");
end

if (FRAME_TIME_OVERRIDE ~= 0) then
   VIDEO_FRAME_TIME = FRAME_TIME_OVERRIDE;
end

if (ASPECT_RATIO_WIDTH) and (ASPECT_RATIO_HEIGHT) then
   AR_X = ASPECT_RATIO_WIDTH;
   AR_Y = ASPECT_RATIO_HEIGHT;
end

AUDIO_BUFFER_SIZE = 500000;
VIDEO_BUFFER_SIZE = 4000000; -- jpeg decoder needs a bigger buffer than its AVC counterpart
VIDEO_STREAM_ID = 4;
AUDIO_STREAM_ID = 1;

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

-- Set up the output port
if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end

if (VIDEO_ONLY == 0) then
   if (SAMPLERATE == 8000) then
      board.setAudioSampleFreq({port=0, freq=16000});
      sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=2048000});
      sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=2048000});
   else
      board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
   end
end
-- Don't set Target Latency Values to test bypassing logic
TARGET_LATENCY_TOLERANCE = TARGET_LATENCY_TOLERANCE or 0; -- in milliseconds
TARGET_LATENCY_UNDERFLOW = TARGET_LATENCY_UNDERFLOW or 0;  -- in milliseconds
TARGET_LATENCY_OVERFLOW  = TARGET_LATENCY_OVERFLOW or 0;   -- in milliseconds

-- Set 1 to start client as third party client
START_AS_THIRD_PARTY_CLIENT = START_AS_THIRD_PARTY_CLIENT or 0;

if (START_AS_THIRD_PARTY_CLIENT == 1) then
    -- Set number of channels
    if (AUDIO_STEREO == 1) then
        NO_OF_CHANNELS = NO_OF_CHANNELS or 2;
    else
        NO_OF_CHANNELS = NO_OF_CHANNELS or 1;
    end

    -- Audio Codec name
    if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QAC) then
        AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "aac";
    elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q711)then
    -- Make is same as g711 A-Law or g711 U-Law as used on server side
      AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "g711u";
    elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q726)then
        AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "g726";
        ABITRATE = ABITRATE  or 40000;
    elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QPCM) then
        AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "pcm";
    elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QMA) then
        AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "mpa";
    end

    -- Video Client Port number
    TP_VIDEO_CLIENT_PORT = TP_VIDEO_CLIENT_PORT or 2048;

    -- Audio Client Port number
    TP_AUDIO_CLIENT_PORT = TP_VIDEO_CLIENT_PORT + 1;
end

if (STREAMING > 0) then
   -- ave frame size = bitrate/fps
   if (FORMAT == "720p") then
      fps = 60;
   elseif (FORMAT == "1080i") then
      fps = 30;
   elseif (FORMAT == "480i") then
      fps = 30;
   elseif (FORMAT == "576i") then
      fps = 30;
   end
   maxBitrate = 200000000; -- bits/sec
   aveFrameSize = maxBitrate / 8 / fps;
   aveFrameSize = aveFrameSize * 2;
   if (aveFrameSize < 1048576) then
      aveFrameSize = 1048576;
   end
   rdef = string.format("sysctl -w net.core.rmem_default=%d", aveFrameSize);
   rmax = string.format("sysctl -w net.core.rmem_max=%d", aveFrameSize);
   -- We modified "SO_RCVBUF" size using following Linux commands so that
   -- we can receive higher bitrate streams without any packet drops:
   os.execute(rdef);
   os.execute(rmax);
end


avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC, vtype=VIDEO_CODEC,
			      audioBufferSize=AUDIO_BUFFER_SIZE, videoBufferSize=VIDEO_BUFFER_SIZE, 
			      maxWidth=HSIZE, maxHeight=VSIZE, ioMode=MEMORY_XFER_MODE});
rs0 = sysctl0:createReadStreamer({name="rs0"});

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});

if (AUDIO_STEREO == 1) then
   avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
else
   avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
end

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});

avdec0:setParam({param=Q_AVDEC_CMP_JPEG_TICKS_PER_FRAME, value=VIDEO_FRAME_TIME});
-- if n/m = 2/3, drop 1 incoming DATA token for every 3
-- if n = m, don't drop at all
-- n must be <= m
avdec0:setParam({param=Q_AVDEC_CMP_JPEG_FRAME_DROP_RATIO_N, value=FRAME_DROP_RATIO_N});
avdec0:setParam({param=Q_AVDEC_CMP_JPEG_FRAME_DROP_RATIO_M, value=FRAME_DROP_RATIO_M});
avdec0:setParam({param=Q_AVDEC_CMP_JPEG_ASPECT_RATIO_WIDTH, value=AR_X});
avdec0:setParam({param=Q_AVDEC_CMP_JPEG_ASPECT_RATIO_HEIGHT, value=AR_Y});

if (FORMAT == "576i") then
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_FIELD_ORDER, value=Q_AVDEC_CFP_VIDEO_FIELD_ORDER_FORCE_BOTTOM_FIRST});
end

avdec0:activateCfg();

-- set video hw params
if (FORMAT == "480i") then
  avoc0:setOutput480i();
elseif (FORMAT == "576i") then
  avoc0:setOutput576i();
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});

avoc0:activateCfg();

avsn0:bindVideoInput({src=avdec0});

if (VIDEO_ONLY == 0) then
  avsn0:bindAudioInput({src=avdec0});

  if (AUDIO_STEREO == 1) then
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
  else
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
  end
  avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
  avoc0:enableAudioChannel({ch=ach0, enable=1});
  avoc0:activateAudioChannelCfg();
else
  AUDIO_STREAM_ID = nil; 
end

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

avoc0:activateVideoChannelCfg();

avoc0:start();

rs0:bindDecoder({dec=avdec0});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=SHOWUPDATE});

if (STREAMING == 0) then
   if (VIDEO_ONLY == 1) then
      bsr0 = rs0:open({type=JPEG_READER_TYPE, file=INFILE, srcId={VIDEO_STREAM_ID}});
   else
      bsr0 = rs0:open({type=JPEG_READER_TYPE, file=INFILE, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
   end
elseif (STREAMING == 1) then
   bsr0 = rs0:open({type="udp", file=string.format("port=%s,bufferlen=%s,qbox=%s", PORT, UDP_BUFFERSIZE, UDP_QBOX),
		      srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
  elseif (STREAMING == 2) then
    if (START_AS_THIRD_PARTY_CLIENT == 1) then
      if (VIDEO_ONLY == 0) then
        if (AUDIO_CODEC_NAME == "g726") then
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tbr=%d,tlt=%d,tlo=%d,tlu=%d",
                   "mjpg", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,ABITRATE,
                    TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                    srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        else
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tlt=%d,tlo=%d,tlu=%d",
            "mjpg", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,
            TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
            srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        end
      else
        bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tvp=%d,tlt=%d,tlo=%d,tlu=%d",
               "mjpg", TP_VIDEO_CLIENT_PORT,
                TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
      end
    else
      -- Not a third party client
      bsr0 = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,cpt=%d",
                     SERVER_IP, SERVER_PORT, RTP_DATA_PROTO, SESSION_ID,
                     TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW, CLIENT_PORT),
                     srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    end

end
rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});

avsn0:start();
avdec0:start();
-- allow time for avoc/avsn to setup completely before sending jpeg frames
-- however, when STREAMING > 0, we want to instantly display the jpeg frames
if (STREAMING == 0) then
   os.execute("sleep 2");
end
rs0:play({bsr=bsr0});

function done()
   rs0:close({bsr=bsr0});
   avdec0:stop();
   avdec0:waitForVideoEOS()
   avsn0:stop();
end

function next()
   rs0:bindDecoder({dec=avdec0});
   if (STREAMING == 0) then
      if (VIDEO_ONLY == 1) then
	 bsr0 = rs0:open({type=JPEG_READER_TYPE, file=INFILE, srcId={VIDEO_STREAM_ID}});
      else
	 bsr0 = rs0:open({type=JPEG_READER_TYPE, file=INFILE, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
      end
   elseif (STREAMING == 1) then
      bsr0 = rs0:open({type="udp", file=string.format("port=%s,bufferlen=%s,qbox=%s", PORT, UDP_BUFFERSIZE, UDP_QBOX),
			 srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
  elseif (STREAMING == 2) then
    if (START_AS_THIRD_PARTY_CLIENT == 1) then
      if (VIDEO_ONLY == 0) then
        if (AUDIO_CODEC_NAME == "g726") then
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tbr=%d,tlt=%d,tlo=%d,tlu=%d",
                   "mjpg", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,ABITRATE,
                    TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                    srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        else
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tlt=%d,tlo=%d,tlu=%d",
            "mjpg", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,
            TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
            srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        end
      else
        bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tvp=%d,tlt=%d,tlo=%d,tlu=%d",
               "mjpg", TP_VIDEO_CLIENT_PORT,
                TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
      end
    else
      -- Not a third party client
      bsr0 = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,cpt=%d",
                     SERVER_IP, SERVER_PORT, RTP_DATA_PROTO, SESSION_ID,
                     TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW, CLIENT_PORT),
                     srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    end
   end
   avsn0:start();
   avdec0:start();
   if (STREAMING == 0) then
      os.execute("sleep 2");
   end
   rs0:play({bsr=bsr0});
end
   

function stop()
   rs0:close({bsr=bsr0});
   avdec0:stop();
   avsn0:stop();
   rs0:unbindDecoder({dec=avdec0});
end
   
function q()
   avoc0:query();
   avsn0:query();
   rs0:query();
   rs0:show();
   avdec0:query();
end

