-- Assume we're using 720p component live video input

VIN = "component";
VOUT = "component";
FORMAT = "720p";

AVC_BITRATE = 6000000;
MJPEG_BITRATE = 5000000;

AVC_ENC_OUTPUT_BUFFER = 3000000;
MJPEG_ENC_OUTPUT_BUFFER = 4000000;

AVC_GUARD_BAND_PER_MB = 200;
MJPEG_GUARD_BAND_PER_MB = 200;

GOP_STRUCTURE = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
FRAME_STRUCTURE = "frame";
PROFILE = "high";

OUTFILE_AVC0 = "test_avc_720.h4v";
OUTFILE_AVC1 = "test_avc_480.h4v";
OUTFILE_RAW_MJPEG = "test_mjpeg.mjpeg";

PREVIEW = 1;
STREAMING = 0;

HSIZE = 1280;  
VSIZE = 720;
MAX_PME_DOWNSCALE = 4;
TF_LUMA_BYPASS = 0;
TF_CHROMA_BYPASS = 1;
VIN_STC = 1500;
VIN_OUTPUT_STC = VIN_STC;
NUM_VIN_FRAMES = 10;
NUM_AVC_REFERENCES = 10;
AVSN_DELAY = 8;
AVC_LEVEL = 50;
HSTART_PIXEL = 291;
VSIZE16 = 16*math.ceil(VSIZE/16)

AVC_STREAMID = 2;
MJPEG_STREAMID = 3;

-- mjpeg config
MJPEG_ENC_INTERVAL = 18000; -- on a 90khz clock, this is 5 FPS
UVC_MODE = 0;
UVC_FRAME_INTERVAL = 2000000;
UVC_PACKET_SIZE = 1024;

-- Function to record AVC for n seconds
function recordAVC(outputfile)
   ws:open({type="file", file=outputfile, sid = {AVC_STREAMID}});
   
   if (PREVIEW == 1) then
      avsn0:start();
   end
   
   avcenc0:bindVideoInput({src=lvpp0});
   avcenc0:record();
   
   lvpp0:start();
end

-- Function to stop recording AVC
function stopAVC()
   lvpp0:flush();
   avcenc0:waitForLastBlock();
   avcenc0:stop();
   avcenc0:unbindVideoInput();
   avsn0:stop();
end

-- Function to record MJPEG for n seconds
function recordMJPEG(outputfile)
   outFileStr = string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_RAW_MJPEG, 1);
   ws:open({type="jpeg", file=outFileStr, sid={MJPEG_STREAMID}});
   
   if (PREVIEW == 1) then
      avsn0:start();
   end

   jpegenc0:bindVideoInput({src=lvpp0});
   jpegenc0:record();
   lvpp0:start();
end

-- Function to stop MJPEG recording
function stopMJPEG()
   lvpp0:flush();
   jpegenc0:stop();
   jpegenc0:unbindVideoInput();
   avsn0:stop();
end

-- Function to change resolution on the fly. 
function changeRes(newW, newH, AR_X, AR_Y)
   avoc0:setParam({param=Q_AVOC_CMP_VID_0_SCALING_ENABLE, value=0}); 
   avoc0:activateCfg();
   os.execute("sleep 5");
   print("change resolution to width = ,height =  ...\n",newW,newH);
   lvpp0:setParam({param=Q_LVPP_CMP_PICT_WIDTH, value=newW});
   lvpp0:setParam({param=Q_LVPP_CMP_PICT_HEIGHT, value=newH});
   lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=AR_X});
   lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=AR_Y}); 
   lvpp0:activateCfg();
end

-- Function to run a test case
function testRun()
   print("\n**** Starting a test run ****\n");

   print("Recording AVC for 5 seconds...\n");
   recordAVC(OUTFILE_AVC0);
   os.execute("sleep 5");
   stopAVC();
   print("AVC stopped\n");

   changeRes(640, 480, 8, 9);

   print("\nRecording AVC for 5 seconds...\n");
   recordAVC(OUTFILE_AVC1);
   os.execute("sleep 5");
   stopAVC();
   print("AVC stopped\n");
   
   changeRes(HSIZE, VSIZE, 1, 1);
   
   print("\nRecording MJPEG for 5 seconds...\n");
   recordMJPEG(OUTFILE_RAW_MJPEG);
   os.execute("sleep 5");
   stopMJPEG();
   printf("MJPEG stopped\n");

   print("Test run done\n");
end

function query() 
  avcenc0:query()
  jpegenc0:query()
  lvpp0:query()
  if (PREVIEW == 1) then
    avoc0:query()
    avsn0:query()
  end
  sysctl0:query()
  ws:query()
end

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

if (PREVIEW == 1) then
   board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end
sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
board.setVideoInput({port=0, format=FORMAT, mode=VIN});

-- writestreamer
ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=0});
ws:start();

-- lvpp
lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16, 
                            numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, 
                            maxPMEDSFactor=MAX_PME_DOWNSCALE, maxDeintMode=0});

Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 4, 4)
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=25});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});

lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=VIN_STC});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});

lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=TF_LUMA_BYPASS});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=TF_CHROMA_BYPASS});

lvpp0:activateCfg();

-- AVC encoder
avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, 
				numreferenceframes=NUM_AVC_REFERENCES, outputbuffersize=AVC_ENC_OUTPUT_BUFFER});
avcenc_720p60(avcenc0, PROFILE, AVC_LEVEL, AVC_BITRATE, GOP_STRUCTURE, FRAME_STRUCTURE);
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=1});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, value=AVC_GUARD_BAND_PER_MB});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_MIN_FRAME_DURATION, value=VIN_OUTPUT_STC});
avcenc0:activateVideoEncCfg();

avcenc0:setStreamID({streamid=AVC_STREAMID});
avcenc0:setAckInterval({interval=3});


-- MJPEG encoder
jpegenc0 = sysctl0:jpegencCreate({name="jpegenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, outputbuffersize=MJPEG_ENC_OUTPUT_BUFFER});

jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=MJPEG_ENC_INTERVAL});
jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=0});
jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=-1});
jpegenc0:setParam({param=Q_JPEGENC_CMP_BITSTREAM_BUFFER, value=1});
jpegenc0:setParam({param=Q_JPEGENC_CMP_AU_GUARD_BAND_PER_MB, value=MJPEG_GUARD_BAND_PER_MB});
jpegenc0:setParam({param=Q_JPEGENC_CMP_ENABLE_UVC, value=UVC_MODE});
jpegenc0:setParam({param=Q_JPEGENC_CMP_UVC_FRAME_INTERVAL, value=UVC_FRAME_INTERVAL});
jpegenc0:setParam({param=Q_JPEGENC_CMP_UVC_PACKET_SIZE, value=UVC_PACKET_SIZE});
jpegenc0:activateCfg();

jpegenc0:setAckInterval({interval=1});
jpegenc0:setStreamID({streamid=MJPEG_STREAMID});
jpegenc0:bitrateControl({bitrate=MJPEG_BITRATE,behavior=0});


if (PREVIEW == 1) then
   avoc0 = sysctl0:avocCreate({name="avoc0"});
   avsn0 = sysctl0:avsnCreate({name="avsn0"});
   avoc0:setOutput720p60();
   avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
   avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});

   avoc0:setParam({param=Q_AVOC_CMP_VID_OUTPUT_TICKS, value=VIN_STC});

   avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
   avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
   avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
   
   avoc0:activateCfg();

   vch0 = avoc0:addVideoChannel();
   avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
   avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
   avoc0:activateVideoChannelCfg();

   avsn0:bindVideoInput({src=lvpp0});
   avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=0});
   avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=VIN_OUTPUT_STC * -3});
   avsn0:activateCfg();

   avcenc0:activateVideoEncCfg();
end

if (PREVIEW == 1) then
  avoc0:start();
end

testRun();