--[[

This script is used to encode Motion-JPEG from live video source.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video input mode
-- "hdmi"
-- "component"
-- "svideo" 
-- "composite" 
VIN             = VIN or "svideo";

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "component" Component output
-- "svideo" SVIDEO output (note composite is driven at same time on output)
VOUT            = VOUT or "svideo";

-- Input Video format
-- "1080i"
-- "1080i50"
-- "720p"
-- "720p50"
-- "480i"
-- "576i"
INPUT_FORMAT    = INPUT_FORMAT or "480i";

-- Output Video format
-- "1080i"
-- "1080i50"
-- "720p"
-- "720p50"
-- "480i"
-- "576i"
OUTPUT_FORMAT    = OUTPUT_FORMAT or "480i";

SHOWRATE = SHOWRATE or 0;

-- Allow video only encode
VIDEO_ONLY = VIDEO_ONLY or 1;

--  To enable the encoded bitstream output
BITSTREAM_BUFFER = BITSTREAM_BUFFER or 1;

-- Controls the audio encoder type
--   "AAC-LC"
--   "ER-AAC-LD"
--   "MPEG-LAYER-2"
--   "G711-A-LAW"
--   "G711-MU-LAW" 
--   "G711" (defaults to mu-law) 
--   "G722"
--   "G726"
--   "G728"
--   "PCM"
AUDIO_ENCODER = AUDIO_ENCODER or "AAC-LC";

-- Audio sample rate (may be overridden depending on codec)
SAMPLERATE = SAMPLERATE or 32000;

-- Audio stereo/mono (may be overridden depending on codec)
AUDIO_STEREO = AUDIO_STEREO or 1;

-- Audio bitrate (may be overridden depending on codec)
if(AUDIO_STEREO) then
   ABITRATE = ABITRATE or 64000;
else
   ABITRATE = ABITRATE or 56000;
end

-- Audio frame size (may be overridden depending on codec)
AUDIO_FRAME_SIZE = AUDIO_FRAME_SIZE or 256;

-- Output file name
OUTFILE_RAW_MJPEG = OUTFILE_RAW_MJPEG or nil; -- test.mjpeg
OUTFILE_QBX_MJPEG = OUTFILE_QBX_MJPEG or nil; -- test.qbx

-- Encoding interval in 90090 ticks/s.
-- -1 means encoding every incoming frames, ie, the encoding frame rate
-- would be the same as the incoming video source.
ENC_INTERVAL    = ENC_INTERVAL or -1;

-- Controls live vs no preview
-- 0 : no preview
-- 1 : live
-- 2 : full-dulpex preview
PREVIEW         = PREVIEW or 1;

-- Only use it when doing full-duplex preview (PREVIEW = 2)
MJPEG_PIPE_NAME = MJPEG_PIPE_NAME or "__jpeg_pipe.mjpeg";

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- 0 is no streaming, 1 for UDP, 2 for RTP
STREAMING       = STREAMING or 0;

-- Enable this flag (changes to 1) to use UDP for board-to-board streaming
-- In board-to-board mode, make sures the JPEG stream id's of encoder and decoder match
UDP_BOARD_TO_BOARD = UDP_BOARD_TO_BOARD or 0;

-- Stream IDs
JPEG_STREAM_ID = JPEG_STREAM_ID or 4;
AUDIO_STREAM_ID = AUDIO_STREAM_ID or 1;

-- UDP related settings
HOST            = HOST or "10.32.84.181";
PORT            = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768; 

-- RTP/RTSP Settings
-- ip address of rtp server.
SERVER_IP = SERVER_IP or "NULL"; -- NULL means find automatically
-- port of rtp server
SERVER_PORT = SERVER_PORT or 554;
-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test";
-- rtp server mode
SERVER_MODE = SERVER_MODE or 0;
-- udp mtu size
UDP_MTU_SIZE = UDP_MTU_SIZE or 1472;
-- tcp mtu size
TCP_MTU_SIZE = TCP_MTU_SIZE or 1472;

-- Set 1 to create third party client
STREAM_TO_THIRD_PARTY_CLIENT = STREAM_TO_THIRD_PARTY_CLIENT or 0;

-- Third party client Ip address
THIRD_PARTY_CLIENT_IP = THIRD_PARTY_CLIENT_IP or "10.32.84.197"

-- Third party client Port
THIRD_PARTY_CLIENT_PORT = THIRD_PARTY_CLIENT_PORT or 2048;

-- Guard Band Size per MB
GUARD_BAND_PER_MB = GUARD_BAND_PER_MB or 200;

-- Converts the output stream format to USB Video Class compatible
UVC_MODE = UVC_MODE or 0;

-- Derived Channel Settings --
-- Use derived channel (NVPP) to upscale or downscale the live video source
DERIVED_CHANNEL_ENABLE = DERIVED_CHANNEL_ENABLE or 0;
DERIVED_HEIGHT         = DERIVED_HEIGHT         or 288; -- CIF
DERIVED_WIDTH          = DERIVED_WIDTH          or 352; --
DERIVED_STC            = DERIVED_STC            or 3003;
DERIVED_CROP_ENABLE    = DERIVED_CROP_ENABLE    or 0;
DERIVED_CROP_X         = DERIVED_CROP_X         or 0;
DERIVED_CROP_Y         = DERIVED_CROP_Y         or 0;
DERIVED_CROP_H         = DERIVED_CROP_H         or 288;
DERIVED_CROP_W         = DERIVED_CROP_W         or 352;

-- Bitstream blocks transfer size
-- Gather from 1 to 5 blocks of encoded frames and send them to the host as one event.
BLOCK_XFER_SIZE = BLOCK_XFER_SIZE or 1;

-- QMED box
QMED_ENABLE = QMED_ENABLE or 0;

--------------------------------
-- Bitrate control parameters --
--------------------------------
-- Enable bitrate control
BITRATE_CONTROL = BITRATE_CONTROL or 1;

-- Bitrate
BITRATE         = BITRATE or 5000000;

-- Bitrate controller's behavior
-- 0: conservative. The default behavior. Bitrate control will maintain
-- a minimum acceptable quality of the encoded frames. You can't have
-- bitrate lower than a certain value, but you can be sure of the quality
-- of the output frames. Allowed quality factor range: 25-100.
-- 1: aggressive. In this behavior, bitrate control will not care about
-- the quality of the encoded frames. When given a very low bitrate, it
-- will drive the quality of the frames to be as low as needed to hit the
-- target bitrate. Allowed quality factor range: 0-100 (full range).
-- 2: customized. This behavior allow two new commands, 
-- Q_JPEGENC_CMD_SET_MIN_QUALITY_FACTOR, and
-- Q_JPEGENC_CMD_SET_MAX_QUALITY_FACTOR to be used.
BITRATE_BEHAVIOR = BITRATE_BEHAVIOR or 0;

--
-- Burn-In parameters
--
FONT_SIZE = FONT_SIZE or 16;
FONT_FILE = FONT_FILE or "/var/mobiapp/fonts/MGSysFont16.bin";
BURNIN_ENABLE = BURNIN_ENABLE or 0;
BURNIN_UPDATE_MODE = BURNIN_UPDATE_MODE or 0;
BURNIN_X = BURNIN_X or 160;
BURNIN_Y = BURNIN_Y or 80;
BURNIN_FRAME_NUM_ENABLE = BURNIN_FRAME_NUM_ENABLE or 1;
BURNIN_FRAME_NUM_POS = BURNIN_FRAME_NUM_POS or 20;
BURNIN_FRAME_NUM_L = BURNIN_FRAME_NUM_L or 0;
BURNIN_FRAME_NUM_H = BURNIN_FRAME_NUM_H or 30;
BURNIN_STR_LEN = BURNIN_STR_LEN or 24;
BURNIN_TEXT = BURNIN_TEXT or "0123456789abcdef'-?,.;:!";

BURNIN_AUTO_TIME_ENABLE = BURNIN_AUTO_TIME_ENABLE or 0;
BURNIN_AUTO_TIME_FPS = BURNIN_AUTO_TIME_FPS or 30
BURNIN_AUTO_TIME_X = BURNIN_AUTO_TIME_X or 320;
BURNIN_AUTO_TIME_Y = BURNIN_AUTO_TIME_Y or 176;
BURNIN_AUTO_TIME_DROPTIME = BURNIN_AUTO_TIME_DROPTIME or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- Encodes elementary MJPEG stream (ie, no qbox headers) or not.
-- This value only affects streaming over UDP (ie, STREAMING == 1)
-- Note: VLC needs elementary MJPEG stream
--       Board-to-board streaming requires qbox MJPEG stream
MJPEG_ELEM      = MJPEG_ELEM or 0;

-- Initial Quality factor, 0 (worst) - 100 (best).
-- If bitrate control is turned off, this value will be used for the quality 
-- factors for all encoded JPEG frames.
-- If bitrate control is turned on, this value will be ignored.
QFACTOR         = QFACTOR or 35;

-- Encoding start index. Use it to skip beginning incoming video frames
ENC_START_INDEX = ENC_START_INDEX or 0;

-- Max number of frames to be encoded
-- 0 means don't encode at all; encoder sit idle
-- -1 means to encode forever
ENC_NUM         = ENC_NUM or -1;

-- Max number of output frames for AV Decoder (full-duplex preview)
MAX_NUM_OUTPUT_FRAME = 17;

-- packet size must be less than 1500. 768 is recommended.
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;

-- UMA and Send file options
ENABLE_UMA = ENABLE_UMA or 1;
ENABLE_SENDFILE = ENABLE_SENDFILE or 1;

-- not sure now
AVSN_DELAY = -1;

if (UDP_BOARD_TO_BOARD == 1) then
   MJPEG_ELEM = 0; -- board-to-board requires qbox streaming
   STREAMING = 1;  -- UDP
   BITSTREAM_BUFFER = 1;
end

UVC_FRAME_INTERVAL = UVC_FRAME_INTERVAL or 2000000;
UVC_PACKET_SIZE    = UVC_PACKET_SIZE    or 256;

if (PREVIEW == 2) then
   BITSTREAM_BUFFER = 1;
end

if (INPUT_FORMAT == "720p") then
  HSIZE = 1280;  
  VSIZE = 720;
  ENC_OUTPUT_BUFFER = 4194304;
  INTERLACED = 0;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 4;
  PME_HSCALE = 4;
  PME_VSCALE = 4;
  if (VIN == "component") then
    START_HPIXEL = 291;
  else
    START_HPIXEL = 251;
  end
  START_VPIXEL = 25;
  AR_X = 1;
  AR_Y = 1;
  FIELD_OFFSET = 563;
  STC = 1500;
  ACK_INTERVAL = 1;
elseif (INPUT_FORMAT == "720p50") then
  HSIZE = 1280;  
  VSIZE = 720;
  ENC_OUTPUT_BUFFER = 4194304;
  INTERLACED = 0;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 4;
  PME_HSCALE = 4;
  PME_VSCALE = 4;
  if (VIN == "component") then
    START_HPIXEL = 291;
  else
    START_HPIXEL = 251;
  end
  START_VPIXEL = 25;
  AR_X = 1;
  AR_Y = 1;
  FIELD_OFFSET = 563;
  STC = 1800;
  ACK_INTERVAL = 1; 
elseif (INPUT_FORMAT == "1080i") then
  HSIZE = 1920;  
  VSIZE = 1080;
  ENC_OUTPUT_BUFFER = 6000000;
  INTERLACED = 1;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 4;
  PME_HSCALE = 4;
  PME_VSCALE = 4;
  if (VIN == "component") then
    START_HPIXEL = 235;
  else
    START_HPIXEL = 193;
  end
  START_VPIXEL = 20;
  AR_X = 1;
  AR_Y = 1;
  FIELD_OFFSET = 563;
  STC = 3003;
  ACK_INTERVAL = 1;
elseif (INPUT_FORMAT == "1080i50") then
  HSIZE = 1920;  
  VSIZE = 1080;
  ENC_OUTPUT_BUFFER = 6000000;
  INTERLACED = 1;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 4;
  PME_HSCALE = 4;
  PME_VSCALE = 4;
  if (VIN == "component") then
    START_HPIXEL = 235;
  else
    START_HPIXEL = 193;
  end
  START_VPIXEL = 20;
  AR_X = 1;
  AR_Y = 1;
  FIELD_OFFSET = 563;
  STC = 3600;
  ACK_INTERVAL = 1;
elseif (INPUT_FORMAT == "480i") then
  HSIZE = 720;  
  VSIZE = 480;
  ENC_OUTPUT_BUFFER = 3145728;
  INTERLACED = 1;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 1;
  PME_HSCALE = 2;
  PME_VSCALE = 4;
  START_HPIXEL = 0;
  START_VPIXEL = 22;
  AR_X = 8;
  AR_Y = 9;
  FIELD_OFFSET = 263;
  STC = 3003;
  ACK_INTERVAL = 3;
elseif (INPUT_FORMAT == "576i") then
  HSIZE = 720;
  VSIZE = 576;
  ENC_OUTPUT_BUFFER = 3145728;
  INTERLACED = 1;
  NUM_VIN_FRAMES = 11;
  MAX_PME_DOWNSCALE = 1;
  PME_HSCALE = 2;
  PME_VSCALE = 4;
  START_HPIXEL = 0;
  START_VPIXEL = 24;
  AR_X = 16;
  AR_Y = 15;
  FIELD_OFFSET = 313;
  STC = 3600;
  ACK_INTERVAL = 3;
else
  error("Invalid input format " .. tostring(INPUT_FORMAT))
end

if (OUTPUT_FORMAT == "720p") then
  DISPLAY_WIDTH = 1280;  
  DISPLAY_HEIGHT = 720;
  DISPLAY_STC = 1500;
elseif (OUTPUT_FORMAT == "720p50") then
  DISPLAY_WIDTH = 1280;  
  DISPLAY_HEIGHT = 720;
  DISPLAY_STC = 1800;
elseif (OUTPUT_FORMAT == "1080i") then
  DISPLAY_WIDTH = 1920;  
  DISPLAY_HEIGHT = 1080;
  DISPLAY_STC = 3003;
  MAX_NUM_OUTPUT_FRAME = 8;
elseif (OUTPUT_FORMAT == "1080i50") then
  DISPLAY_WIDTH = 1920;  
  DISPLAY_HEIGHT = 1080;
  DISPLAY_STC = 3600;
  MAX_NUM_OUTPUT_FRAME = 8;
elseif (OUTPUT_FORMAT == "480i") then
  DISPLAY_WIDTH = 720;  
  DISPLAY_HEIGHT = 480;
  DISPLAY_STC = 3003;
elseif (OUTPUT_FORMAT == "576i") then
  DISPLAY_WIDTH = 720;  
  DISPLAY_HEIGHT = 576;
  DISPLAY_STC = 3600;
else
  error("Invalid output format " .. tostring(OUTPUT_FORMAT))
end

VSIZE16 = 16*math.ceil(VSIZE/16)

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

if (INPUT_FORMAT == "1080i") or (INPUT_FORMAT == "1080i50") then
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00000ba0, interleave=1});
elseif (INPUT_FORMAT == "720p") or (INPUT_FORMAT == "720p50") then
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
elseif (INPUT_FORMAT == "480i") then
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
elseif (INPUT_FORMAT == "576i") then
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
end

if (OUTPUT_FORMAT == "1080i") or (OUTPUT_FORMAT == "1080i50") then
  board.setVideoOutput({port=0, format=OUTPUT_FORMAT, mode=VOUT, sync=vout_sync});
elseif (OUTPUT_FORMAT == "720p") or (OUTPUT_FORMAT == "720p50") then
  board.setVideoOutput({port=0, format=OUTPUT_FORMAT, mode=VOUT});
elseif (OUTPUT_FORMAT == "480i") then
  board.setVideoOutput({port=0, format=OUTPUT_FORMAT, mode=VOUT});
elseif (OUTPUT_FORMAT == "576i") then
  board.setVideoOutput({port=0, format=OUTPUT_FORMAT, mode=VOUT});
end

if (DERIVED_CHANNEL_ENABLE) then
   sysctl0:setVPortMode({port=1, live=0});
end

board.setVideoInput({port=0, format=INPUT_FORMAT, mode=VIN});

-- audio parameters and board setup for audio
if (VIDEO_ONLY == 0) then
   -- setup audio parameters with over-rides as necessary
   QAC_AOT = 2;
   Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
   if (AUDIO_ENCODER == "AAC-LC") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 1024;
      QAC_AOT = 2;
   elseif (AUDIO_ENCODER == "ER-AAC-LD") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 512;
      QAC_AOT = 23;
   elseif (AUDIO_ENCODER == "MPEG-LAYER-2") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QMA;
      AUDIO_FRAME_SIZE = 1152;
   elseif (AUDIO_ENCODER == "G711-A-LAW") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_A_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G711-MU-LAW") or (AUDIO_ENCODER == "G711") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G722") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q722;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 16000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "PCM") then
      AUDIO_CODEC =  Q_AUDENC_CFP_CODEC_QPCM;
      SAMPLERATE = 16000;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G726") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q726;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      if (ABITRATE ~= 40000) and (ABITRATE ~= 32000) and (ABITRATE ~= 24000) and (ABITRATE ~= 16000) then
         ABITRATE = 40000; -- set usable default
      end
   elseif (AUDIO_ENCODER == "G728") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q728;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 16000;
   else
      error("Invalid audio encoder " .. tostring(AUDIO_ENCODER))
   end
   board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
end

if (STREAMING > 0) then
  -- We modified "SO_SNDBUF" size to 1024*1024 using following Linux commands so that
  -- we can send the stream without any packet drops:
  os.execute("sysctl -w net.core.wmem_default=1048576");
  os.execute("sysctl -w net.core.wmem_max=1048576");
end

sysctl0:setVerboseLevel({param=3});
ws = sysctl0:createWriteStreamer({name="ws0", uma=ENABLE_UMA, sendfile=ENABLE_SENDFILE});
ws:showrate({enable=SHOWRATE});
ws:start();

-- load burnin font
BurnInFontAddr = sysctl0:allocBurnInFont({fontsize=FONT_SIZE});
sysctl0:loadBurnInFont({fontaddr=BurnInFontAddr, fontsize=FONT_SIZE, filename=FONT_FILE});

-- Setup LVPP
lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16, 
  numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, maxPMEDSFactor=MAX_PME_DOWNSCALE,
  maxDeintMode=0});

-- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
Setup_VPP(lvpp0, HSIZE, VSIZE, INTERLACED, 0, PME_HSCALE, PME_VSCALE);

-- disable temporal filter
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=1});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=1});

lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=FIELD_OFFSET});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=START_VPIXEL});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=START_HPIXEL});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=STC});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=STC});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=AR_X});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=AR_Y});

if (INPUT_FORMAT == "480i") or (INPUT_FORMAT == "576i") then
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_DEINT_MODE, value=0});
end

if (VIDEO_ONLY == 0) then
  lvpp0:setParam({param=Q_LVPP_CMP_AV_SYNCH_ENABLE, value=1});
end

lvpp0:activateCfg();

-- Setup NVPP
if (DERIVED_CHANNEL_ENABLE == 1) then
   nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=HSIZE, maxHeight=VSIZE, 
				 maxPMEDSFactor=PME_HSCALE, numVidFrame=NUM_VIN_FRAMES, maxDeintMode=0});
   Setup_VPP(nvpp0, HSIZE, VSIZE, 0, 0, PME_HSCALE, PME_VSCALE);
   
   nvpp0:setParam({param=Q_NVPP_CMP_PICT_WIDTH, value=DERIVED_WIDTH});
   nvpp0:setParam({param=Q_NVPP_CMP_PICT_HEIGHT, value=DERIVED_HEIGHT});

   if (DERIVED_CROP_ENABLE == 1) then
      nvpp0:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X, value=DERIVED_CROP_X});
      nvpp0:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y, value=DERIVED_CROP_Y});
      nvpp0:setParam({param=Q_NVPP_CMP_CROP_WIDTH,    value=DERIVED_CROP_W});
      nvpp0:setParam({param=Q_NVPP_CMP_CROP_HEIGHT,   value=DERIVED_CROP_H});
   end


   nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_FRAME, value=DERIVED_STC});
   nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=DERIVED_STC});
   nvpp0:setParam({param=Q_NVPP_CMP_PMEDS_EN, value=0});
   nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_X, value=AR_X});
   nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y, value=AR_Y});
   nvpp0:setParam({param=Q_NVPP_CMP_HW_BYPASS_TFILTY, value=1});
   nvpp0:setParam({param=Q_NVPP_CMP_HW_BYPASS_TFILTC, value=1});
   nvpp0:setParam({param=Q_NVPP_CMP_HW_BYPASS_MB_STAT, value=0});
   nvpp0:setParam({param=Q_NVPP_CMP_PMEDS_FRAME_MODE, value=1});
   nvpp0:setParam({param=Q_NVPP_CMP_DEINT_MODE, value=0});
   nvpp0:setParam({param=Q_NVPP_CMP_INPUT_DECIMATION, value=1});
   nvpp0:activateCfg();
   nvpp0:bindInput({src=lvpp0});
end

-- create and configure audio input
if (VIDEO_ONLY == 0) then   
   -- setup audio input
   sain0 = sysctl0:sainCreate({name="sain0",port=0});
   if (AUDIO_STEREO == 1) then
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_STEREO});
   else
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_MONO_L});
   end
   sain0:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
   sain0:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
   sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
   sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
   sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
   sain0:activateCfg();

   -- create and bind audio encoder
   audenc0 = sysctl0:audencCreate({name="audenc0", ws=ws, codecType=AUDIO_CODEC});
   audenc0:setStreamID({streamid=AUDIO_STREAM_ID});
   audenc0:bindAudioInput({src=sain0});

   -- configure audio encoder
   if (AUDIO_STEREO == 1) then
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_STEREO});
   else
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_MONO});
   end
   audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=SAMPLERATE});
   audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
   audenc0:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
   --codec specific configuration
   audenc0:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q711_LAW});
   audenc0:setParam({param=Q_AUDENC_CMP_QAC_AUDIO_OBJECT_TYPE, value=QAC_AOT});
   audenc0:activateCfg();
end


if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0 = sysctl0:avocCreate({name="avoc0"});
  avsn0 = sysctl0:avsnCreate({name="avsn0"});
  if (OUTPUT_FORMAT == "1080i") then
    avoc0:setOutput1080i60({sync=vout_sync});
  elseif (OUTPUT_FORMAT == "1080i50") then
    avoc0:setOutput1080i50({sync=vout_sync});
  elseif (OUTPUT_FORMAT == "720p") then
    avoc0:setOutput720p60();
  elseif (OUTPUT_FORMAT == "720p50") then
    avoc0:setOutput720p50();
  elseif (OUTPUT_FORMAT == "480i") then
    avoc0:setOutput480i();
  elseif (OUTPUT_FORMAT == "576i") then
    avoc0:setOutput576i();
  end
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=DISPLAY_WIDTH});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=DISPLAY_HEIGHT});

  avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
  avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
  avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
  avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});

  avoc0:activateCfg();

  if (PREVIEW == 1) then
    avsn0:bindVideoInput({src=lvpp0});
    avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=DISPLAY_STC * -3});
  elseif (PREVIEW == 2) then
    AUDIO_BUFFER_SIZE = 1000000;
    VIDEO_BUFFER_SIZE = 4000000;
    avdec0 = sysctl0:avdecCreate({name="avdec0", 
				  atype=Q_AVDEC_CFP_AUDIO_CODEC_QAC, 
				  vtype=Q_AVDEC_CFP_VIDEO_CODEC_JPEG, 
				  audioBufferSize=AUDIO_BUFFER_SIZE,
				  videoBufferSize=VIDEO_BUFFER_SIZE, 
				  maxWidth=HSIZE, maxHeight=VSIZE,
                                  numoutputframes=MAX_NUM_OUTPUT_FRAME});
    rs0 = sysctl0:createReadStreamer({name="rs0"});
    rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=0});
   
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=JPEG_STREAM_ID});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=DISPLAY_STC});
   
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=500000});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_THRESHOLD, value=500000});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_MARGIN, value=5000});
   
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});
   
    avdec0:setParam({param=Q_AVDEC_CMP_JPEG_TICKS_PER_FRAME, value=DISPLAY_STC});
    avdec0:setParam({param=Q_AVDEC_CMP_JPEG_FRAME_DROP_RATIO_N, value=1});
    avdec0:setParam({param=Q_AVDEC_CMP_JPEG_FRAME_DROP_RATIO_M, value=1});
   
    avdec0:activateCfg();
    rs0:bindDecoder({dec=avdec0});
    avsn0:bindVideoInput({src=avdec0});
    avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=DISPLAY_STC * -1 * AVSN_DELAY});
  end
  avsn0:activateCfg();

  vch0 = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=DISPLAY_WIDTH});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=DISPLAY_HEIGHT});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=DISPLAY_WIDTH});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=DISPLAY_HEIGHT});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
  avoc0:activateVideoChannelCfg();

  if (VIDEO_ONLY == 0) then
    -- Echo audio to output
    avsn0:bindAudioInput({src=sain0});
    if (AUDIO_STEREO == 1) then
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
    else
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
    end
    avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
    avoc0:enableAudioChannel({ch=ach0, enable=1});
    avoc0:activateAudioChannelCfg();
  end  

end


jpegenc0 = sysctl0:jpegencCreate({name="jpegenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, outputbuffersize=ENC_OUTPUT_BUFFER});

if (DERIVED_CHANNEL_ENABLE == 1) then
   jpegenc0:bindVideoInput({src=nvpp0});
else
   jpegenc0:bindVideoInput({src=lvpp0});
end

--os.execute("rm -f "..FILENAME);
if (PREVIEW == 2) then
  os.execute("mkfifo -m 664 "..MJPEG_PIPE_NAME);
  outFileStr = string.format("basefilename=%s,single=0,jelem=%d", MJPEG_PIPE_NAME, 1);
  ws:open({type="jpeg", file=outFileStr, sid={JPEG_STREAM_ID}});
end

if (OUTFILE_RAW_MJPEG) then
  outFileStr = string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_RAW_MJPEG, 1);
  ws:open({type="jpeg", file=outFileStr, sid={JPEG_STREAM_ID}});
end
if (OUTFILE_QBX_MJPEG) then
  outFileStr = string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_QBX_MJPEG, 0);
  ws:open({type="jpeg", file=outFileStr, sid={JPEG_STREAM_ID}});
end
    
if(STREAMING == 1) then
  ws:open({type="net", 
           file=string.format("host=%s,port=%d,protocol=udp,max=%d,jelem=%d", 
                              HOST, PORT, MAX_PACKET_SIZE, MJPEG_ELEM), 
           sid = {JPEG_STREAM_ID}});
elseif(STREAMING == 2) then
  if (STREAM_TO_THIRD_PARTY_CLIENT == 1) then
    if (VIDEO_ONLY == 1) then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=none,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID}});
    elseif (AUDIO_ENCODER == "AAC-LC") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=aac,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    elseif (AUDIO_ENCODER == "G711-A-LAW") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=g711a,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    elseif (AUDIO_ENCODER == "G711-MU-LAW") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=g711u,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    elseif (AUDIO_ENCODER == "G711") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=g711u,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    elseif (AUDIO_ENCODER == "G726") then
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=g726,mod=%d,msz=1472,cli=1,cip=%s,cpt=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
    else
      -- Audio format not supported in RTP streaming so just stream video
        print("Audio encoder " .. AUDIO_ENCODER .. " not supported in RTP streaming so just streaming video\n");
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=mjpeg,aud=none,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT), sid = {VIDEOSTREAMID}});
    end
  else
    if (VIDEO_ONLY == 1) then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=none,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID}});
    elseif (AUDIO_ENCODER == "AAC-LC") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=aac,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID, AUDIO_STREAM_ID}});
    elseif (AUDIO_ENCODER == "G711-A-LAW") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=g711a,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID, AUDIO_STREAM_ID}});
    elseif (AUDIO_ENCODER == "G711-MU-LAW") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=g711u,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID, AUDIO_STREAM_ID}});
    elseif (AUDIO_ENCODER == "G711") then
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=g711u,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID, AUDIO_STREAM_ID}});
    elseif (AUDIO_ENCODER == "G726") then
    ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=g726,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID, AUDIO_STREAM_ID}});
    else
      print("Audio encoder " .. AUDIO_ENCODER .. " not supported in RTP streaming so just streaming video\n");
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,aud=none,ums=%d,tms=%d",
					    SERVER_IP, SERVER_PORT , SESSION_ID , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE), sid = {JPEG_STREAM_ID}});
    end
  end
end


jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=ENC_INTERVAL});
jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=ENC_START_INDEX});
jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=ENC_NUM});
jpegenc0:setParam({param=Q_JPEGENC_CMP_BITSTREAM_BUFFER, value=BITSTREAM_BUFFER});
jpegenc0:setParam({param=Q_JPEGENC_CMP_AU_GUARD_BAND_PER_MB, value=GUARD_BAND_PER_MB});
jpegenc0:setParam({param=Q_JPEGENC_CMP_ENABLE_UVC, value=UVC_MODE});
jpegenc0:setParam({param=Q_JPEGENC_CMP_UVC_FRAME_INTERVAL, value=UVC_FRAME_INTERVAL});
jpegenc0:setParam({param=Q_JPEGENC_CMP_UVC_PACKET_SIZE, value=UVC_PACKET_SIZE});
jpegenc0:setParam({param=Q_JPEGENC_CMP_ENABLE_QMED, value=QMED_ENABLE});

jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_ENABLE, value=BURNIN_ENABLE});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FONT_SIZE, value=FONT_SIZE});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_UPD_MODE, value=BURNIN_UPDATE_MODE});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_X, value=BURNIN_X});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_Y, value=BURNIN_Y});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FRAME_NUM_ENABLE, value=BURNIN_FRAME_NUM_ENABLE});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FRAME_NUM_POS, value=BURNIN_FRAME_NUM_POS});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FRAME_NUM_L, value=BURNIN_FRAME_NUM_L});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_FRAME_NUM_H, value=BURNIN_FRAME_NUM_H});
jpegenc0:setBurnInParam({index=0, param=Q_JPEGENC_CMP_BURNIN_STR_LEN, value=BURNIN_STR_LEN});
jpegenc0:setBurnInText({index=0, text=BURNIN_TEXT});
jpegenc0:activateCfg();

jpegenc0:setBurnInParam({index=1, param=Q_JPEGENC_CMP_BURNIN_FONT_ADDR, value=BurnInFontAddr});
jpegenc0:setBurnInParam({index=1, param=Q_JPEGENC_CMP_BURNIN_FONT_SIZE, value=FONT_SIZE});

jpegenc0:setBurnInAutoTime({  index=1, 
			      enable=BURNIN_AUTO_TIME_ENABLE, 
			      fps=BURNIN_AUTO_TIME_FPS,
			      x=BURNIN_AUTO_TIME_X, 
			      y=BURNIN_AUTO_TIME_Y, 
			      droptime=BURNIN_AUTO_TIME_DROPTIME
			   });

jpegenc0:setAckInterval({interval=ACK_INTERVAL});
jpegenc0:setStreamID({streamid=JPEG_STREAM_ID});
jpegenc0:setQualityFactor({qualityfactor=QFACTOR});
jpegenc0:setTransferSize({size=BLOCK_XFER_SIZE});

if (BITRATE_CONTROL == 1) then
   jpegenc0:bitrateControl({bitrate=BITRATE,behavior=BITRATE_BEHAVIOR});
end

if (UVC_MODE == 1) then
   -- enable DMA word-boundary mode
   os.execute("echo 1 0x014 4 0x57e7d > /proc/driver/qcc/writebid");
end

-- start output
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0:start();
end

if (PREVIEW == 2) then
   bsr0 = rs0:open({type="jpeg", file=MJPEG_PIPE_NAME, srcId={JPEG_STREAM_ID}, codecId={JPEG_STREAM_ID}});
end

if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avsn0:start();
end

if (PREVIEW == 2) then
   avdec0:start();
   rs0:play({bsr=bsr0});
end

if (VIDEO_ONLY == 0) then
  audenc0:record();
end

jpegenc0:record();
if (DERIVED_CHANNEL_ENABLE == 1) then
   nvpp0:start();
end
lvpp0:start();

if (VIDEO_ONLY == 0) then
  sain0:start();
end

function done()
   lvpp0:flush();
   if (BITSTREAM_BUFFER == 1) then
      ws:close({type="jpeg", file=outFileStr});
   end
end

function jstop()
   lvpp0:flush();
   jpegenc0:stop();
   if(VIDEO_ONLY == 0) then
      sain0:flush();
   end
   avsn0:stop();
end

function jbind()
   if (DERIVED_CHANNEL_ENABLE == 1) then
      jpegenc0:bindVideoInput({src=nvpp0});
   else
      jpegenc0:bindVideoInput({src=lvpp0});
   end
end

function junbind()
   jpegenc0:unbindVideoInput();
end

function disableBitrateControl()
   jpegenc0:bitrateControl({disable=1});
end

function newBitrate(br, bhr, rst)
   jpegenc0:bitrateControl({bitrate=br, behavior=bhr, reset=rst});
end

function minMaxQF(min, max)
   jpegenc0:setMinQualityFactor({qualityfactor=min});
   jpegenc0:setMaxQualityFactor({qualityfactor=max});
end

function lumaQM(i, v)
   jpegenc0:setQuantMatrix({type=Q_JPEGENC_CFP_QUANT_MATRIX_TYPE_LUMA, index=i, value=v});
   --jpegenc0:activateQuantMatrix(); -- use it to activate all changes to the matrix at once
end

function chromaQM(i, v)
   jpegenc0:setQuantMatrix({type=Q_JPEGENC_CFP_QUANT_MATRIX_TYPE_CHROMA, index=i, value=v});
   --jpegenc0:activateQuantMatrix(); -- use it to activate all changes to the matrix at once
end

function q()
  if ((PREVIEW == 1) or (PREVIEW == 2)) then
    avoc0:query();
    avsn0:query();
  end
  ws:query();
  lvpp0:query();
  if (DERIVED_CHANNEL_ENABLE == 1) then
     nvpp0:query();
  end
  if (PREVIEW == 2) then
    avdec0:query();
  end
  if (VIDEO_ONLY == 0) then
    audenc0:query();
  end
  jpegenc0:query();
end

function shot(n)
   jpegenc0:snapshot({shots=n});
end

