--[[

This script is used to encode Motion-JPEG from multiple 480i 
live video sources.

--]]

--------------------------------
-- General control parameters --
--------------------------------

-- Video input can be svideo or composite
VIN = VIN or "svideo";

-- Video output is set to HDMI for 1080i mode and svideo for 480i mode
VOUT = VOUT or "hdmi";

-- Video output can be 1080i or 480i (note composite also is driven in svideo)
FORMAT = FORMAT or '1080i';

-- Number of live encoded channels
ENCCH = ENCCH or 4;

-- Width and height of each live encoded stream
ENCW            = ENCW or 720;
ENCH            = ENCH or 480;

-- To enable the encoded bitstream output
BITSTREAM_BUFFER    = BITSTREAM_BUFFER or {};
BITSTREAM_BUFFER[1] = 1;
BITSTREAM_BUFFER[2] = 1;
BITSTREAM_BUFFER[3] = 1;
BITSTREAM_BUFFER[4] = 1;
for i = 5, ENCCH do
   BITSTREAM_BUFFER[i] = BITSTREAM_BUFFER[4];
end

-- Output file names
OUTFILE_RAW_MJPEG    = OUTFILE_RAW_MJPEG or {};
OUTFILE_RAW_MJPEG[1] = OUTFILE_RAW_MJPEG[1] or nil;
OUTFILE_RAW_MJPEG[2] = OUTFILE_RAW_MJPEG[2] or nil;
OUTFILE_RAW_MJPEG[3] = OUTFILE_RAW_MJPEG[3] or nil;
OUTFILE_RAW_MJPEG[4] = OUTFILE_RAW_MJPEG[4] or nil;
for i = 5, ENCCH do
   OUTFILE_RAW_MJPEG[i] = nil;
end

OUTFILE_QBX_MJPEG    = OUTFILE_QBX_MJPEG or {};
OUTFILE_QBX_MJPEG[1] = OUTFILE_QBX_MJPEG[1] or nil;
OUTFILE_QBX_MJPEG[2] = OUTFILE_QBX_MJPEG[2] or nil;
OUTFILE_QBX_MJPEG[3] = OUTFILE_QBX_MJPEG[3] or nil;
OUTFILE_QBX_MJPEG[4] = OUTFILE_QBX_MJPEG[4] or nil;
for i = 5, ENCCH do
   OUTFILE_QBX_MJPEG[i] = nil;
end

-- Encoding interval in 90090 ticks/s.
-- -1 means encoding every incoming frames, ie, the encoding frame rate
-- would be the same as the incoming video source.
ENC_INTERVAL    = ENC_INTERVAL or 9009;

-- Controls live vs no preview
-- 0 : no preview
-- 1 : live
-- reconstructed preview does not exist for mjpeg encoder
PREVIEW         = PREVIEW or 1;

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- 0 is no streaming, 1 for UDP
STREAMING       = STREAMING or {};
STREAMING[1]    = STREAMING[1] or 1;
STREAMING[2]    = STREAMING[2] or 1;
STREAMING[3]    = STREAMING[3] or 1;
STREAMING[4]    = STREAMING[4] or 1;
for i = 5, ENCCH do
   STREAMING[i] = STREAMING[4];
end
HOST            = HOST or "10.32.82.86";
PORT            = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;

-- Guard Band Size per MB
GUARD_BAND = GUARD_BAND or {};
GUARD_BAND[1] = GUARD_BAND[1] or 100;
GUARD_BAND[2] = GUARD_BAND[2] or 100;
GUARD_BAND[3] = GUARD_BAND[3] or 100;
GUARD_BAND[4] = GUARD_BAND[4] or 100;
for i = 5, ENCCH do
   GUARD_BAND[i] = GUARD_BAND[4];
end

--------------------------------
-- Bitrate control parameters --
--------------------------------
-- Enable bitrate control
BITRATE_CONTROL    = BITRATE_CONTROL or {};
BITRATE_CONTROL[1] = 1;
BITRATE_CONTROL[2] = 1;
BITRATE_CONTROL[3] = 1;
BITRATE_CONTROL[4] = 1;
for i = 5, ENCCH do
   BITRATE_CONTROL[i] = BITRATE_CONTROL[4];
end

-- Bitrate
BITRATE		  = BITRATE or {};
BITRATE[1]        = 1500000;
BITRATE[2]        = 1500000;
BITRATE[3]        = 1500000;
BITRATE[4]        = 1500000;
for i = 5, ENCCH do
   BITRATE[i] = BITRATE[4];
end

-- Bitrate controller's behavior
-- 0: conservative. The default behavior. Bitrate control will maintain
-- a minimum acceptable quality of the encoded frames. You can't have
-- bitrate lower than a certain value, but you can be sure of the quality
-- of the output frames. Allowed quality factor range: 25-100.
-- 1: aggressive. In this behavior, bitrate control will not care about
-- the quality of the encoded frames. When given a very low bitrate, it
-- will drive the quality of the frames to be as low as needed to hit the
-- target bitrate. Allowed quality factor range: 0-100 (full range).
-- 2: customized. This behavior allow two new commands, 
-- Q_JPEGENC_CMD_SET_MIN_QUALITY_FACTOR, and
-- Q_JPEGENC_CMD_SET_MAX_QUALITY_FACTOR to be used.
BITRATE_BEHAVIOR = BITRATE_BEHAVIOR or {};
BITRATE_BEHAVIOR[1] = 0;
BITRATE_BEHAVIOR[2] = 0;
BITRATE_BEHAVIOR[3] = 0;
BITRATE_BEHAVIOR[4] = 0;
for i = 5, ENCCH do
   BITRATE_BEHAVIOR[i] = BITRATE_BEHAVIOR[4];
end

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- Encodes elementary MJPEG stream (ie, no qbox headers) or not.
-- This value only affects streaming over UDP (ie, STREAMING == 1)
-- Note: VLC needs elementary MJPEG stream.
--       Board-to-board streaming requires qbox MJPEG stream.
MJPEG_ELEM    = MJPEG_ELEM or {};
MJPEG_ELEM[1] = 1;
MJPEG_ELEM[2] = 1;
MJPEG_ELEM[3] = 1;
MJPEG_ELEM[4] = 1;
for i = 5, ENCCH do
   MJPEG_ELEM[i] = MJPEG_ELEM[4];
end

-- Encoding start index. Use it to skip beginning incoming video frames
ENC_START_INDEX    = ENC_START_INDEX or {};
ENC_START_INDEX[1] = 0;
ENC_START_INDEX[2] = 0;
ENC_START_INDEX[3] = 0;
ENC_START_INDEX[4] = 0;
for i = 5, ENCCH do
   ENC_START_INDEX[i] = ENC_START_INDEX[4];
end

-- Max number of frames to be encoded
-- 0 means don't encode at all; encoder sit idle
-- -1 means to encode as much as possible
ENC_NUM    = ENC_NUM or {}
ENC_NUM[1] = -1;
ENC_NUM[2] = -1;
ENC_NUM[3] = -1;
ENC_NUM[4] = -1;
for i = 5, ENCCH do
   ENC_NUM[i] = ENC_NUM[4];
end

-- Initial Quality factor, 0 (worst) - 100 (best).
-- If bitrate control is turned off, this value will be used for the quality 
-- factors for all encoded JPEG frames.
-- If bitrate control is turned on, this value will be ignored.
QFACTOR         = QFACTOR or {};
QFACTOR[1] = 35;
QFACTOR[2] = 35;
QFACTOR[3] = 35;
QFACTOR[4] = 35;
for i = 5, ENCCH do
   QFACTOR[i] = QFACTOR[4];
end

if (FORMAT == "1080i") then
  DISPLAYWIDTH = 1920;  
  DISPLAYHEIGHT = 1080;
  OUTW = 720;
  OUTH = 480;
elseif (FORMAT == "480i") then
  DISPLAYWIDTH = 720;
  DISPLAYHEIGHT = 480;
  OUTW=360;
  OUTH=240;
end

ENC_OUTPUT_BUFFER = 3145728;
INTERLACED = 1;
NUM_VIN_FRAMES = 11;
MAX_PME_DOWNSCALE = 1;
VIDEO_CONTROL = 0x00101000;
START_HPIXEL = 0;
START_VPIXEL = 22;
AR_X = 8;
AR_Y = 9;
FIELD_OFFSET = 263;
STC = 3003;
ACK_INTERVAL = 1;
JPEGSTREAMID = 3;

-- Number of rendered images
OUTCH=4;

-- Width and height of the output grid
OUTGRIDW=2;
OUTGRIDH=2;

sysctl0:setVPortMode({port=0, live=1, interlaced=INTERLACED, bottomfieldfirst=1, videocontrol=VIDEO_CONTROL, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end
if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end

board.setVideoInput({port=0, format='480i', mode=VIN});
sysctl0:setVerboseLevel({param=3});

-- To use a 1MB socket buffer
ONE_MB_BUF = 0;
for i = 1, 4 do
   if (STREAMING[i] > 0) then
      ONE_MB_BUF = 1;
   end
end

if (ONE_MB_BUF == 1) then
  -- We modified "SO_SNDBUF" size to 1024*1024 using following Linux commands so that
  -- we can send the stream without any packet drops:
  os.execute("sysctl -w net.core.wmem_default=1048576");
  os.execute("sysctl -w net.core.wmem_max=1048576");
end

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (DISPLAYHEIGHT-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (DISPLAYWIDTH-(OUTGRIDW*OUTW))/2;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
if (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
elseif (FORMAT == "480i") then
  avoc0:setOutput480i();
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
--avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

if (OUTCH > ENCCH) then
  OUTCH = ENCCH;
end

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=ENCW, 
  maxHeight=ENCH, numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, 
  maxPMEDSFactor=MAX_PME_DOWNSCALE});

ws = sysctl0:createWriteStreamer({name="ws0", uma=1, sendfile=1, queuesize=5000});
ws:showrate({enable=0});
ws:start();

Setup_VPP(lvpp0, ENCW, ENCH, INTERLACED, 1, 4, 4)

--lvpp0:crop({cropWidth=720, cropHeight=480, horzOffset=0, vertOffset=0});
--lvpp0:size({outputWidth=ENCW, outputHeight=ENCH});

-- disable temporal filter
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=1});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=1});

lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=FIELD_OFFSET});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=START_VPIXEL});
lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=START_HPIXEL});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=STC});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=STC});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=AR_X});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=AR_Y});
--lvpp0:setParam({param=Q_LVPP_CMP_PMEDS_EN, value=1});

lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_DEINT_MODE, value=0});

lvpp0:activateCfg();

-- CREATE ENCODERS
jpegenc = {};
avsnenc = {};

for i = 1,ENCCH do
  jpegenc[i] = sysctl0:jpegencCreate({name="jpegenc"..i, ws=ws, maxwidth=ENCW, maxheight=ENCH, 
				      outputbuffersize=ENC_OUTPUT_BUFFER});
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"});
  jid = JPEGSTREAMID + i;
  jpegenc[i]:bindVideoInput({src=lvpp0});

  if (OUTFILE_RAW_MJPEG[i]) then
     outFileStr = string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_RAW_MJPEG[i], 1);
     ws:open({type="jpeg", file=outFileStr, sid={jid}});
  end

  if (OUTFILE_QBX_MJPEG[i]) then
     outFileStr = string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_QBX_MJPEG[i], 0);
     ws:open({type="jpeg", file=outFileStr, sid={jid}});
  end

  if (STREAMING[i] == 1) then
     ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,jelem=%d,max=%d", 
         HOST, PORT+i-1, MJPEG_ELEM[i], MAX_PACKET_SIZE), sid = {jid}});
  end
  
  jpegenc[i]:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=ENC_INTERVAL});
  jpegenc[i]:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=ENC_START_INDEX[i]});
  jpegenc[i]:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=ENC_NUM[i]});
  jpegenc[i]:setParam({param=Q_JPEGENC_CMP_BITSTREAM_BUFFER, value=BITSTREAM_BUFFER[i]});
  jpegenc[i]:setParam({param=Q_JPEGENC_CMP_AU_GUARD_BAND_PER_MB, value=GUARD_BAND[i]});
  jpegenc[i]:activateCfg();

  jpegenc[i]:setAckInterval({interval=ACK_INTERVAL});
  jpegenc[i]:setStreamID({streamid=jid});
  jpegenc[i]:setQualityFactor({qualityfactor=QFACTOR[i]});

  if (BITRATE_CONTROL[i] == 1) then
     jpegenc[i]:bitrateControl({bitrate=BITRATE[i],behavior=BITRATE_BEHAVIOR[i]});
  end
end

vch = {};
x = 0;
y = 0;
for i = 1,OUTCH do
  vch[i] = avoc0:addVideoChannel();

  if PREVIEW == 1 then
     avsnenc[i]:bindVideoInput({src=lvpp0});
  end
  avoc0:setVideoChannelSource({ch=vch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- START

avoc0:start();

for i = 1,OUTCH do
  avsnenc[i]:start();
end

for i = 1,ENCCH do
  jpegenc[i]:record();
end
lvpp0:start();

----------------------
-- Helper functions --
----------------------
function brc_set(n,newRate,newBhr,rst)
   jpegenc[n]:bitrateControl({bitrate=newRate, behavior=newBhr, reset=rst});
end

function brc_off(n)
   jpegenc[n]:bitrateControl({disable=1});
end

function done()
   lvpp0:flush();
   if (BITSTREAM_BUFFER == 1) then
      for i = 1, ENCCH do
	 if (OUTFILE_RAW_MJPEG[i]) then
	    ws:close({type="jpeg", file=string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_RAW_MJPEG[i], 1)});
	 end
	 if (OUTFILE_QBX_MJPEG[i]) then
	    ws:close({type="jpeg", file=string.format("basefilename=%s,single=0,jelem=%d", OUTFILE_QBX_MJPEG[i], 0)});
	 end
      end
   end
end

function q()
   lvpp0:query();
   for i = 1,ENCCH do
      jpegenc[i]:query();
   end
   ws:query();
end
