--[[

This script is used to split 1 NTSC stream into 8 or 4 frames.The input is scaled using NVPPs and
displayed on the screen. The number of channels can be set to either 4 or 8 depending on the 
output desired. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------
-- Change no of channels to 4 if you want 4 frames displayed on the screen 

NO_OF_CHANNELS = NO_OF_CHANNELS or 8;

BITRATE = BITRATE or {};

-- Number of live encoded channels
if NO_OF_CHANNELS == 8 then
	ENCCH = ENCCH or 8;
else
	ENCCH = ENCCH or 4;
end

-- 0 for nothing, 1 for reconstructed frames, 2 for live -- 2 not working yet
PREVIEW = PREVIEW or 1;

-- Host IP and base port for udp live stream (port, port+1, port+2...)
STREAMING = STREAMING or 0;
HOST = HOST or "192.168.40.38";
PORT = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;

-- Width and height of each live encoded stream

if NO_OF_CHANNELS == 8 then
	ENCW = ENCW or 352;
	ENCH = ENCH or 240;
else 
	ENCW = ENCW or 720;
	ENCH = ENCH or 480;
end

-- Bitrate of each live encoded stream
for i=0,ENCCH-1 do
  BITRATE[i] = 500000;
end

-- Put override bitrates here
BITRATE[1] = BITRATE[1] or 100000;

-- Number of rendered images and their scaled size
if NO_OF_CHANNELS == 8 then
	OUTCH = OUTCH or 8;
	OUTW  =	OUTW or	352;
	OUTH  =	OUTH or 240;
else
	OUTCH = OUTCH or 4;
	OUTW  =	OUTW or	720;
	OUTH  =	OUTH or 480;
end
	
-- Width and height of the output grid
if NO_OF_CHANNELS == 8 then
	OUTGRIDW = OUTGRIDW or 4;
	OUTGRIDH = OUTGRIDH or 2;
else
	OUTGRIDW =OUTGRIDW or 2;
	OUTGRIDH =OUTGRIDH or 2;
end

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF = 1;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

BITSTREAM_OUTPUT=1;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=1, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format='1080i', mode='hdmi', sync=vout_sync});
board.setVideoInput({port=0, format='480i', mode='svideo'});

sysctl0:setVerboseLevel({param=7});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

ENCCH = ENCCH - 1;
OUTCH = OUTCH - 1;

if (OUTCH > ENCCH) then OUTCH = ENCCH end

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=ENCW, maxHeight=ENCH, numVidFrame=6, maxPMEDSFactor=1});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

--dofile(os.getenv("QUARC_ROOT").."/merlinsw/exthost/scripts/libvpp.lua");
lvpp0:crop({cropWidth=720, cropHeight=480, horzOffset=0, vertOffset=0});
lvpp0:size({outputWidth=ENCW, outputHeight=ENCH});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=1});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=1});
lvpp0:activateCfg();

-- CREATE ENCODERS
nvppenc = {};
avsnenc = {};
videoStreamId = {};

-- AVC SETUP
--dofile(os.getenv("QUARC_ROOT").."/merlinsw/exthost/scripts/libavcenc.lua");

for i = 0, ENCCH do
  nvppenc[i] = sysctl0:nvppCreate({name="nvpp0", maxWidth=ENCW, maxHeight=ENCH, numVidFrame=6,maxPMEDSFactor=1});
  
  -- delta of 16 between live and derived channels
  videoStreamId = i + 2;

  -- workaround - crop instead of scale 2nd channel
  Setup_VPP(nvppenc[i], ENCW, ENCH, 1, 1, 2, 2)
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PMEDS_EN,            value=1});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  nvppenc[i]:activateCfg();
  nvppenc[i]:bindInput({src=lvpp0});
 end
  
 for i = 0,ENCCH do
  
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"});
 end


vch = {};
x = 0;
y = 0;
for i = 0,OUTCH do
  vch[i] = avoc0:addVideoChannel();

  if PREVIEW == 1 then
      avsnenc[i]:bindVideoInput({src=nvppenc[i]});

  elseif PREVIEW == 2  then
      print("binding video input to live\n");
      avsnenc[i]:bindVideoInput({src=lvpp0});
  end
  avoc0:setVideoChannelSource({ch=vch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  print("configured avoc");
  
  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- START
avoc0:start();

for i = 0,OUTCH do
  avsnenc[i]:start();
end

for i = 0,ENCCH do
   nvppenc[i]:start();
end

lvpp0:start();

function done()
  lvpp0:flush();
  os.execute("echo flushed");
  for i = 0,ENCCH do
    ws:close({type="h4v", file=string.format("test%d.h4v", i)});
  end
end


function q()
	for i = 0,ENCCH do
		nvppenc[i]:query();
		avsnenc[i]:query();
	end
	lvpp0:query();
	avoc0:query();
end
