--[[

This script first decodes a 1080i MPEG2 video stream, then re-encode it into
AVC format.  The decoded stream will be shown on display.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/hbo_trailer_6k_1920x1080.ts" 

VOUT          = VOUT or "hdmi";

-- hbo_trailer is 17
STREAM_PID    = STREAM_PID or 17;

-- 0 for nothing, 1 for NVPP
PREVIEW       = PREVIEW or 1;

OUTFILE_H4V   = OUTFILE_H4V or nil; --"test.h4v";

OUTFILE_QBX   = OUTFILE_QBX or nil; --"test.qbx";

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- By default, this script is for 1080i streams
FORMAT     = FORMAT or "1080i";
MAX_WIDTH  = MAX_WIDTH or 1920;
MAX_HEIGHT = MAX_HEIGHT or 1080;

-- Audio/Video Stream IDs
AUD_STREAM_ID     = 1;
VID_STREAM_ID_IN  = 0;
VID_STREAM_ID_OUT = 2;

NUM_AVC_REFERENCE = 4;

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
sysctl0:setVPortMode({port=1, live=0});

rs0 = sysctl0:createReadStreamer({name="rs0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, vtype=Q_AVDEC_CFP_VIDEO_CODEC_M2V, vstream=Q_AVDEC_CFP_VIDEO_STREAM_TS,
                              audioBufferSize=100000, videoBufferSize=1000000, maxWidth=MAX_WIDTH, maxHeight=MAX_HEIGHT, numoutputframes=6});
nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=MAX_WIDTH, maxHeight=MAX_HEIGHT});
ws = sysctl0:createWriteStreamer({name="ws0"});
avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=MAX_WIDTH, maxheight=MAX_HEIGHT, numreferenceframes=NUM_AVC_REFERENCE,
                                outputbuffersize=4000000});
avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});

-- AVDEC settings --

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUD_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VID_STREAM_ID_IN});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=16000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=5000});

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_FIELD_ORDER, value=Q_AVDEC_CFP_VIDEO_FIELD_ORDER_AUTO});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TS_PID, value=STREAM_PID});

avdec0:activateCfg();

-- NVPP settings --

-- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
Setup_VPP(nvpp0, MAX_WIDTH, MAX_HEIGHT, 1, 0, 4, 4);
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_X, value=1});
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y, value=1});
nvpp0:setParam({param=Q_NVPP_CMP_FAST_SCALE_MODE, value=1});
nvpp0:activateCfg();
nvpp0:bindInput({src=avdec0});

-- AVC Encoder settings --

-- avcenc_1080i60(avcenc, profile, level, bitrate_qp, gop_structure, picture_coding)
avcenc_1080i60(avcenc0, "high", 40, 4000000, Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP, 2);
avcenc0:setStreamID({streamid=VID_STREAM_ID_OUT});
avcenc0:bindVideoInput({src=nvpp0});

-- Writestreamer settings --

if (OUTFILE_H4V) then
    ws:open({type="h4v", file=OUTFILE_H4V, sid = {VID_STREAM_ID_OUT}});
end

if (OUTFILE_QBX) then
    ws:open({type="file", file=OUTFILE_QBX, sid = {VID_STREAM_ID_OUT}});
end

-- Output settings --

if (PREVIEW == 1) then
    avsn0:bindVideoInput({src=nvpp0});
end

avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=MAX_WIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=MAX_HEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=0});
avoc0:activateCfg();

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=MAX_WIDTH});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=MAX_HEIGHT});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=MAX_WIDTH});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=MAX_HEIGHT});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

avoc0:activateVideoChannelCfg();

rs0:bindDecoder({dec=avdec0});
bsr0 = rs0:open({type="dump", file=INPUTFILENAME, srcId={VID_STREAM_ID_IN}});

avoc0:start();
ws:start();
avcenc0:record();
nvpp0:start();
avsn0:start();

avdec0:start();
rs0:play({bsr=bsr0});


-- Wait for stop command to finish
-- avdec0:waitForVideoEOS()
-- 
-- rs0:close({bsr=bsr0});
-- exit();

